import collections
import typing
from abjad import enums
from abjad import typings
from abjad.core.Chord import Chord
from abjad.core.Leaf import Leaf
from abjad.core.MultimeasureRest import MultimeasureRest
from abjad.core.Note import Note
from abjad.core.Rest import Rest
from abjad.core.Skip import Skip
from abjad.core.Staff import Staff
from abjad.top.inspect import inspect
from abjad.top.new import new
from abjad.top.sequence import sequence
from abjad.utilities.Duration import Duration
from abjad.utilities.Offset import Offset
from abjad.utilities.String import String
from .Spanner import Spanner


# TODO: teach durated beam to respect beam_lone_notes
class Beam(Spanner):
    r"""
    Beam.

    ..  container:: example

        >>> staff = abjad.Staff("c'8 d'8 e'8 f'8 g'2")
        >>> abjad.setting(staff).auto_beaming = False
        >>> abjad.show(staff) # doctest: +SKIP

        ..  docs::

            >>> abjad.f(staff)
            \new Staff
            \with
            {
                autoBeaming = ##f
            }
            {
                c'8
                d'8
                e'8
                f'8
                g'2
            }

        >>> beam = abjad.Beam()
        >>> abjad.attach(beam, staff[:2])
        >>> beam = abjad.Beam()
        >>> abjad.attach(beam, staff[2:4])
        >>> abjad.show(staff) # doctest: +SKIP

        ..  docs::

            >>> abjad.f(staff)
            \new Staff
            \with
            {
                autoBeaming = ##f
            }
            {
                c'8
                [
                d'8
                ]
                e'8
                [
                f'8
                ]
                g'2
            }

    ..  container:: example

        Tweaks beam positions:

        >>> staff = abjad.Staff("c'8 d'8 e'8 f'8 g'2")
        >>> abjad.setting(staff).auto_beaming = False
        >>> beam = abjad.Beam()
        >>> abjad.tweak(beam).positions = (3, 3)
        >>> abjad.attach(beam, staff[:2])
        >>> beam = abjad.Beam()
        >>> abjad.tweak(beam).positions = (3, 3)
        >>> abjad.attach(beam, staff[2:4])
        >>> abjad.show(staff) # doctest: +SKIP

        ..  docs::

            >>> abjad.f(staff)
            \new Staff
            \with
            {
                autoBeaming = ##f
            }
            {
                c'8
                - \tweak positions #'(3 . 3)
                [
                d'8
                ]
                e'8
                - \tweak positions #'(3 . 3)
                [
                f'8
                ]
                g'2
            }

    ..  container:: example

        Spanners can be tagged:

        >>> staff = abjad.Staff("c'8 d'8 e'8 f'8 g'2")
        >>> abjad.setting(staff).auto_beaming = False
        >>> abjad.show(staff) # doctest: +SKIP

        ..  docs::

            >>> abjad.f(staff)
            \new Staff
            \with
            {
                autoBeaming = ##f
            }
            {
                c'8
                d'8
                e'8
                f'8
                g'2
            }

        >>> beam = abjad.Beam()
        >>> abjad.attach(beam, staff[:2], tag='BEAM')
        >>> beam = abjad.Beam()
        >>> abjad.attach(beam, staff[2:4])
        >>> abjad.show(staff) # doctest: +SKIP

        >>> abjad.f(staff)
        \new Staff
        \with
        {
            autoBeaming = ##f
        }
        {
            c'8
            [ %! BEAM
            d'8
            ] %! BEAM
            e'8
            [
            f'8
            ]
            g'2
        }

    """

    ### CLASS VARIABLES ###

    __slots__ = (
        '_beam_lone_notes',
        '_beam_rests',
        '_direction',
        '_durations',
        '_span_beam_count',
        '_stemlet_length',
        )

    _start_command = '['

    _stop_command = ']'

    ### INITIALIZER ###

    def __init__(
        self,
        *,
        beam_lone_notes: bool = None,
        beam_rests: bool = None,
        direction: typing.Union[str, enums.VerticalAlignment] = None,
        durations: typing.Iterable[Duration] = None,
        span_beam_count: int = 1,
        stemlet_length: typings.Number = None,
        ) -> None:
        Spanner.__init__(self)
        direction = String.to_tridirectional_lilypond_symbol(direction)
        if beam_lone_notes is not None:
            beam_lone_notes = bool(beam_lone_notes)
        self._beam_lone_notes = beam_lone_notes
        if beam_rests is not None:
            beam_rests = bool(beam_rests)
        self._beam_rests = beam_rests
        self._direction = direction
        durations = self._coerce_durations(durations)
        self._durations: typing.Tuple[Duration, ...] = durations
        if span_beam_count is not None:
            assert isinstance(span_beam_count, int)
        self._span_beam_count = span_beam_count
        self._stemlet_length = stemlet_length

    ### PRIVATE METHODS ###

    def _add_beam_counts(self, leaf, bundle):
        if (not isinstance(leaf, Leaf) or not self._is_beamable(leaf)):
            left, right = None, None
        elif self._is_exterior_leaf(leaf):
            left, right = self._get_left_right_for_exterior_leaf(leaf)
        elif self._is_just_left_of_gap(leaf):
            left = leaf.written_duration.flag_count
            next_leaf = inspect(leaf).leaf(1)
            if self._is_beamable(
                next_leaf,
                beam_rests=self.beam_rests,
                ):
                right = self.span_beam_count
            else:
                right = 0
        elif self._is_just_right_of_gap(leaf):
            previous_leaf = inspect(leaf).leaf(-1)
            if self._is_beamable(
                previous_leaf,
                beam_rests=self.beam_rests,
                ):
                left = self.span_beam_count
            else:
                left = 0
            right = leaf.written_duration.flag_count
        else:
            assert self._is_interior_leaf(leaf)
            left, right = self._get_left_right_for_interior_leaf(leaf)
        if left is not None:
            string = rf'\set stemLeftBeamCount = {left}'
            bundle.before.commands.append(string)
        if right is not None:
            string = rf'\set stemRightBeamCount = {right}'
            bundle.before.commands.append(string)

    def _add_start_and_stops(self, leaf, bundle):
        if self._is_beamable(leaf, beam_rests=self.beam_rests):
            previous_leaf = leaf._get_leaf(-1)
            next_leaf = leaf._get_leaf(1)
            if (leaf is self[0] or
                not previous_leaf or
                not self._is_beamable(
                    previous_leaf,
                    beam_rests=self.beam_rests,
                    )
                ):
                strings = self._tweaked_start_command_strings()
                bundle.after.spanner_starts.extend(strings)
            if (leaf is self[-1] or
                not next_leaf or
                not self._is_beamable(next_leaf, beam_rests=self.beam_rests)):
                string = self._stop_command_string()
                for spanner_start in bundle.after.spanner_starts:
                    if '[' in spanner_start:
                        bundle.after.spanner_starts.append(string)
                        break
                else:
                    bundle.after.spanner_stops.append(string)

    def _add_stemlet_length(self, leaf, bundle):
        if self.stemlet_length is None:
            return
        if leaf is self[0]:
            parentage = inspect(leaf).parentage()
            staff = parentage.get_first(Staff)
            lilypond_type = staff.lilypond_type
            string = r'\override {}.Stem.stemlet-length = {}'
            string = string.format(lilypond_type, self.stemlet_length)
            bundle.before.commands.append(string)
        if leaf is self[-1]:
            parentage = inspect(leaf).parentage()
            staff = parentage.get_first(Staff)
            lilypond_type = staff.lilypond_type
            string = r'\revert {}.Stem.stemlet-length'
            string = string.format(lilypond_type, self.stemlet_length)
            bundle.before.commands.append(string)

    @staticmethod
    def _coerce_durations(durations) -> typing.Tuple[Duration, ...]:
        durations = durations or []
        assert isinstance(durations, collections.Iterable)
        durations = [Duration(_) for _ in durations]
        return tuple(durations)

    def _copy_keywords(self, new):
        Spanner._copy_keywords(self, new)
        new._beam_lone_notes = self.beam_lone_notes
        new._beam_rests = self.beam_rests
        new._direction = self.direction
        if self.durations is not None:
            new._durations = self.durations[:]
        new._span_beam_count = self.span_beam_count
        new._stemlet_length = self.stemlet_length

    def _fracture_left(self, i):
        self, left, right = Spanner._fracture_left(self, i)
        if self.durations:
            weights = [
                inspect(left).duration(),
                inspect(right).duration(),
                ]
            assert sum(self.durations) == sum(weights)
            split_durations = sequence(self.durations)
            split_durations = split_durations.split(
                weights,
                cyclic=False,
                overhang=False,
                )
            left_durations, right_durations = split_durations
            left._durations = left_durations
            right._durations = right_durations
        return self, left, right

    def _fracture_right(self, i):
        self, left, right = Spanner._fracture_right(self, i)
        if self.durations:
            weights = [
                inspect(left).duration(),
                inspect(right).duration(),
                ]
            assert sum(self.durations) == sum(weights)
            split_durations = sequence(self.durations)
            split_durations = split_durations.split(
                weights,
                cyclic=False,
                overhang=False,
                )
            left_durations, right_durations = split_durations
            left._durations = left_durations
            right._durations = right_durations
        return self, left, right

    def _get_beam_count_lilypond_format_bundle(self, leaf):
        bundle = self._get_basic_lilypond_format_bundle(leaf)
        self._add_stemlet_length(leaf, bundle)
        self._add_beam_counts(leaf, bundle)
        self._add_start_and_stops(leaf, bundle)
        return bundle

    def _get_left_right_for_exterior_leaf(self, leaf):
        # lone
        if self._is_my_only(leaf):
            left, right = None, None
        # first
        elif leaf is self[0] or not leaf._get_leaf(-1):
            left = 0
            right = leaf.written_duration.flag_count
        # last
        elif leaf is self[-1] or leaf._get_leaf(1):
            left = leaf.written_duration.flag_count
            right = 0
        else:
            message = 'leaf must be first or last in spanner.'
            raise ValueError(message)
        return left, right

    def _get_left_right_for_interior_leaf(self, leaf):
        """
        Interior leaves are neither first nor last in spanner.
        Interior leaves may be surrounded by beamable leaves.
        Interior leaves may be surrounded by unbeamable leaves.
        Four cases total for beamability of surrounding leaves.
        """
        import abjad
        previous_leaf = leaf._get_leaf(-1)
        previous_written = previous_leaf.written_duration
        current_written = leaf.written_duration
        next_leaf = leaf._get_leaf(1)
        next_written = next_leaf.written_duration
        previous_flag_count = previous_written.flag_count
        current_flag_count = current_written.flag_count
        next_flag_count = next_written.flag_count
        rest_prototype = (
            abjad.MultimeasureRest,
            abjad.Rest,
            abjad.Skip,
            )
        #print(previous_leaf, leaf, next_leaf)
        # [unbeamable leaf beamable]
        if (not self._is_beamable(
                previous_leaf,
                beam_rests=self.beam_rests,
                ) and
            self._is_beamable(
                next_leaf,
                beam_rests=self.beam_rests,
                )
            ):
            left = current_flag_count
            right = min(current_flag_count, next_flag_count)
        # [beamable leaf unbeamable]
        elif (self._is_beamable(
                previous_leaf,
                beam_rests=self.beam_rests,
                ) and
            not self._is_beamable(
                next_leaf,
                beam_rests=self.beam_rests,
                )
            ):
            left = min(current_flag_count, previous_flag_count)
            right = current_flag_count
        # [unbeamable leaf unbeamable]
        elif (not self._is_beamable(
                previous_leaf,
                beam_rests=self.beam_rests,
                ) and
            not self._is_beamable(
                next_leaf,
                beam_rests=self.beam_rests,
                )
            ):
            left = current_flag_count
            right = current_flag_count
        # [beamable leaf beamable]
        else:
            if isinstance(previous_leaf, rest_prototype):
                left = current_flag_count
            else:
                left = min(current_flag_count, previous_flag_count)
            if isinstance(next_leaf, rest_prototype):
                right = current_flag_count
            else:
                right = min(current_flag_count, next_flag_count)
            if left != current_flag_count and right != current_flag_count:
                left = current_flag_count
        return left, right

    def _get_lilypond_format_bundle(self, leaf):
        if self.durations:
            return self._get_beam_count_lilypond_format_bundle(leaf)
        bundle = self._get_basic_lilypond_format_bundle(leaf)
        if not self._is_beamable(leaf, beam_rests=self.beam_rests):
            return bundle
        beamable_leaf_count = 0
        for leaf_ in self.leaves:
            if self._is_beamable(leaf_, beam_rests=self.beam_rests):
                beamable_leaf_count += 1
        if self.beam_lone_notes or 2 <= beamable_leaf_count:
            previous_leaf = leaf._get_leaf(-1)
            if previous_leaf not in self.leaves:
                previous_leaf = None
            if (previous_leaf in self.leaves and
                self._is_beamable(
                    previous_leaf,
                    beam_rests=self.beam_rests,
                    )):
                previous_leaf_is_beamable = True
            else:
                previous_leaf_is_beamable = False
            next_leaf = leaf._get_leaf(1)
            if (next_leaf in self.leaves and
                self._is_beamable(
                    next_leaf,
                    beam_rests=self.beam_rests,
                    )):
                next_leaf_is_beamable = True
            else:
                next_leaf_is_beamable = False
            start_pieces = []
            stop_piece = None
            if leaf is self[0]:
                if next_leaf_is_beamable or self.beam_lone_notes:
                    start_pieces = self._tweaked_start_command_strings()
            else:
                if (not previous_leaf_is_beamable and
                    (next_leaf_is_beamable or self.beam_lone_notes)):
                    start_pieces = self._tweaked_start_command_strings()
            if leaf is self[-1]:
                if previous_leaf_is_beamable or self.beam_lone_notes:
                    stop_piece = self._stop_command_string()
            else:
                if ((previous_leaf_is_beamable or self.beam_lone_notes) and
                    not next_leaf_is_beamable):
                    stop_piece = self._stop_command_string()
            if start_pieces and stop_piece:
                bundle.after.spanner_starts.extend(start_pieces)
                bundle.after.spanner_starts.append(stop_piece)
            elif start_pieces:
                bundle.after.spanner_starts.extend(start_pieces)
            elif stop_piece:
                bundle.after.spanner_stops.append(stop_piece)
        self._add_stemlet_length(leaf, bundle)
        return bundle

    def _get_span_beam_offsets(self):
        offsets = []
        if self.durations:
            offset = Offset(self.durations[0])
            offsets.append(offset)
            for duration in self.durations[1:]:
                offset = offsets[-1] + duration
                offsets.append(offset)
            offsets.pop()
        return offsets

    @staticmethod
    def _is_beamable(argument, beam_rests=False) -> bool:
        """
        Is true when ``argument`` is a beamable component.

        ..  container:: example

            Without allowing for beamed rests:

            >>> staff = abjad.Staff(r"r32 a'32 ( [ gs'32 fs''32 \staccato f''8 ) ]")
            >>> staff.extend(r"r8 e''8 ( ef'2 )")
            >>> abjad.show(staff) # doctest: +SKIP

            >>> for leaf in staff:
            ...     result = abjad.Beam._is_beamable(leaf)
            ...     print(f'{str(leaf):<8}\t{result}')
            ...
            r32     False
            a'32    True
            gs'32   True
            fs''32  True
            f''8    True
            r8      False
            e''8    True
            ef'2    False

        ..  container:: example

            Allowing for beamed rests:

            >>> staff = abjad.Staff(r"r32 a'32 ( [ gs'32 fs''32 \staccato f''8 ) ]")
            >>> staff.extend(r"r8 e''8 ( ef'2 )")
            >>> abjad.show(staff) # doctest: +SKIP

            >>> for leaf in staff:
            ...     result = abjad.Beam._is_beamable(
            ...         leaf,
            ...         beam_rests=True,
            ...         )
            ...     print(f'{str(leaf):<8}\t{result}')
            ...
            r32	True
            a'32	True
            gs'32	True
            fs''32	True
            f''8	True
            r8	True
            e''8	True
            ef'2	False

        ..  container:: example

            Is true for skips of any duration when ``beam_rests`` is true:

            >>> skip = abjad.Skip((1, 32))
            >>> abjad.Beam._is_beamable(skip, beam_rests=True)
            True

            >>> skip = abjad.Skip((1))
            >>> abjad.Beam._is_beamable(skip, beam_rests=True)
            True

        ..  container:: example

            Is true for rests of any duration when ``beam_rests`` is true:

            >>> rest = abjad.Rest((1, 32))
            >>> abjad.Beam._is_beamable(rest, beam_rests=True)
            True

            >>> rest = abjad.Rest((1))
            >>> abjad.Beam._is_beamable(rest, beam_rests=True)
            True

        """
        if isinstance(argument, (Chord, Note)):
            if 0 < argument.written_duration.flag_count:
                return True
        prototype = (
            MultimeasureRest,
            Rest,
            Skip,
            )
        if beam_rests and isinstance(argument, prototype):
            return True
        return False

    def _is_just_left_of_gap(self, leaf):
        local_stop_offset = self._stop_offset_in_me(leaf)
        span_beam_offsets = self._get_span_beam_offsets()
        if local_stop_offset in span_beam_offsets:
            return True
        return False

    def _is_just_right_of_gap(self, leaf):
        local_start_offset = self._start_offset_in_me(leaf)
        span_beam_offsets = self._get_span_beam_offsets()
        if local_start_offset in span_beam_offsets:
            return True
        return False

    ### PUBLIC PROPERTIES ###

    @property
    def beam_lone_notes(self) -> typing.Optional[bool]:
        r"""
        Is true when beam should include lone notes.

        ..  todo:: Teach durated beams to respect ``beam_lone_notes``.

        ..  container:: example

            Does not beam lone notes:

            >>> staff = abjad.Staff("c'8 d' e' f'")
            >>> abjad.setting(staff).auto_beaming = False
            >>> beam = abjad.Beam()
            >>> abjad.attach(beam, staff[:1])
            >>> abjad.show(staff) # doctest: +SKIP

            ..  docs::

                >>> abjad.f(staff)
                \new Staff
                \with
                {
                    autoBeaming = ##f
                }
                {
                    c'8
                    d'8
                    e'8
                    f'8
                }

            >>> staff = abjad.Staff("c'8 d'4 e'8 f'4 g'8")
            >>> abjad.setting(staff).auto_beaming = False
            >>> beam = abjad.Beam()
            >>> abjad.attach(beam, staff[:])
            >>> abjad.show(staff) # doctest: +SKIP

            ..  docs::

                >>> abjad.f(staff)
                \new Staff
                \with
                {
                    autoBeaming = ##f
                }
                {
                    c'8
                    d'4
                    e'8
                    f'4
                    g'8
                }

        ..  container:: example

            Does beam lone notes:

            >>> staff = abjad.Staff("c'8 d' e' f'")
            >>> abjad.setting(staff).auto_beaming = False
            >>> beam = abjad.Beam(beam_lone_notes=True)
            >>> abjad.attach(beam, staff[:1])
            >>> abjad.show(staff) # doctest: +SKIP

            ..  docs::

                >>> abjad.f(staff)
                \new Staff
                \with
                {
                    autoBeaming = ##f
                }
                {
                    c'8
                    [
                    ]
                    d'8
                    e'8
                    f'8
                }

            >>> staff = abjad.Staff("c'8 d'4 e'8 f'4 g'8")
            >>> abjad.setting(staff).auto_beaming = False
            >>> beam = abjad.Beam(beam_lone_notes=True)
            >>> abjad.attach(beam, staff[:])
            >>> abjad.show(staff) # doctest: +SKIP

            ..  docs::

                >>> abjad.f(staff)
                \new Staff
                \with
                {
                    autoBeaming = ##f
                }
                {
                    c'8
                    [
                    ]
                    d'4
                    e'8
                    [
                    ]
                    f'4
                    g'8
                    [
                    ]
                }

        """
        return self._beam_lone_notes

    @property
    def beam_rests(self) -> typing.Optional[bool]:
        r"""
        Is true when beam should include rests.

        ..  container:: example

            Without beamed rests:

            >>> staff = abjad.Staff("c'8 d'8 r8 f'8 g'8 r4.")
            >>> abjad.setting(staff).auto_beaming = False
            >>> beam = abjad.Beam()
            >>> abjad.attach(beam, staff[:])
            >>> abjad.show(staff) # doctest: +SKIP

            >>> abjad.f(staff)
            \new Staff
            \with
            {
                autoBeaming = ##f
            }
            {
                c'8
                [
                d'8
                ]
                r8
                f'8
                [
                g'8
                ]
                r4.
            }

            Abjad beams no rests.

            LilyPond beams no rests.

        ..  container:: example

            With beamed rests:

            >>> staff = abjad.Staff("c'8 d'8 r8 f'8 g'8 r4.")
            >>> abjad.setting(staff).auto_beaming = False
            >>> beam = abjad.Beam(beam_rests=True)
            >>> abjad.attach(beam, staff[:])
            >>> abjad.show(staff) # doctest: +SKIP

            >>> abjad.f(staff)
            \new Staff
            \with
            {
                autoBeaming = ##f
            }
            {
                c'8
                [
                d'8
                r8
                f'8
                g'8
                r4.
                ]
            }

            Abjad beams all rests.

            LilyPond beams only small-duration rests.

        ..  container:: example

            With beamed rests:

            >>> staff = abjad.Staff("c'8 d'8 r4. f'8 g'8 r8")
            >>> abjad.setting(staff).auto_beaming = False
            >>> beam = abjad.Beam(beam_rests=True)
            >>> abjad.attach(beam, staff[:])
            >>> abjad.show(staff) # doctest: +SKIP

            >>> abjad.f(staff)
            \new Staff
            \with
            {
                autoBeaming = ##f
            }
            {
                c'8
                [
                d'8
                r4.
                f'8
                g'8
                r8
                ]
            }

            Abjad beams all rests.

            LilyPond beams only small-duration rests.

        ..  container:: example

            With beamed skips:

            >>> staff = abjad.Staff("c'8 d'8 s4. f'8 g'8 s8")
            >>> abjad.setting(staff).auto_beaming = False
            >>> beam = abjad.Beam(beam_rests=True)
            >>> abjad.attach(beam, staff[:])
            >>> abjad.show(staff) # doctest: +SKIP

            >>> abjad.f(staff)
            \new Staff
            \with
            {
                autoBeaming = ##f
            }
            {
                c'8
                [
                d'8
                s4.
                f'8
                g'8
                s8
                ]
            }

            Abjad beams all skips.

            LilyPond beams all (internal) skips.

        """
        return self._beam_rests

    @property
    def direction(self) -> typing.Optional[String]:
        """
        Gets direction.
        """
        return self._direction
        
    @property
    def durations(self) -> typing.Tuple[Duration, ...]:
        r"""
        Gets durations.

        ..  container:: example

            Two groups:

            >>> staff = abjad.Staff("c'16 d'16 e'16 f'16")
            >>> durations = [(1, 8), (1, 8)]
            >>> beam = abjad.Beam(durations=durations)
            >>> abjad.attach(beam, staff[:])
            >>> abjad.show(staff) # doctest: +SKIP

            ..  docs::

                >>> abjad.f(staff)
                \new Staff
                {
                    \set stemLeftBeamCount = 0
                    \set stemRightBeamCount = 2
                    c'16
                    [
                    \set stemLeftBeamCount = 2
                    \set stemRightBeamCount = 1
                    d'16
                    \set stemLeftBeamCount = 1
                    \set stemRightBeamCount = 2
                    e'16
                    \set stemLeftBeamCount = 2
                    \set stemRightBeamCount = 0
                    f'16
                    ]
                }

            >>> beam.durations
            (Duration(1, 8), Duration(1, 8))

        ..  container:: example

            Three groups:

            >>> staff = abjad.Staff("c'16 d'16 e'8 f'16 c'16")
            >>> abjad.setting(staff).auto_beaming = False
            >>> durations = [(1, 8), (1, 8), (1, 8)]
            >>> beam = abjad.Beam(
            ...     durations=durations,
            ...     span_beam_count=1,
            ...     )
            >>> abjad.attach(beam, staff[:])
            >>> abjad.show(staff) # doctest: +SKIP

            ..  docs::

                >>> abjad.f(staff)
                \new Staff
                \with
                {
                    autoBeaming = ##f
                }
                {
                    \set stemLeftBeamCount = 0
                    \set stemRightBeamCount = 2
                    c'16
                    [
                    \set stemLeftBeamCount = 2
                    \set stemRightBeamCount = 1
                    d'16
                    \set stemLeftBeamCount = 1
                    \set stemRightBeamCount = 1
                    e'8
                    \set stemLeftBeamCount = 1
                    \set stemRightBeamCount = 2
                    f'16
                    \set stemLeftBeamCount = 2
                    \set stemRightBeamCount = 0
                    c'16
                    ]
                }

            Test ensures that leaf groups format correctly when they contain
            only one leaf.

        """
        return self._durations

    @property
    def span_beam_count(self) -> int:
        r"""
        Gets span beam count.

        ..  container:: example

            Creates a single span beam between adjacent groups in spanner:

            >>> staff = abjad.Staff("c'32 d'32 e'32 f'32")
            >>> durations = [(1, 16), (1, 16)]
            >>> beam = abjad.Beam(
            ...     durations=durations,
            ...     span_beam_count=1,
            ...     )
            >>> abjad.attach(beam, staff[:])
            >>> abjad.show(staff) # doctest: +SKIP

            ..  docs::

                >>> abjad.f(staff)
                \new Staff
                {
                    \set stemLeftBeamCount = 0
                    \set stemRightBeamCount = 3
                    c'32
                    [
                    \set stemLeftBeamCount = 3
                    \set stemRightBeamCount = 1
                    d'32
                    \set stemLeftBeamCount = 1
                    \set stemRightBeamCount = 3
                    e'32
                    \set stemLeftBeamCount = 3
                    \set stemRightBeamCount = 0
                    f'32
                    ]
                }

            >>> beam.span_beam_count
            1

        ..  container:: example

            Creates a double span beam between adjacent groups in spanner:

            >>> staff = abjad.Staff("c'32 d'32 e'32 f'32")
            >>> durations = [(1, 16), (1, 16)]
            >>> beam = abjad.Beam(
            ...     durations=durations,
            ...     span_beam_count=2,
            ...     )
            >>> abjad.attach(beam, staff[:])
            >>> abjad.show(staff) # doctest: +SKIP

            ..  docs::

                >>> abjad.f(staff)
                \new Staff
                {
                    \set stemLeftBeamCount = 0
                    \set stemRightBeamCount = 3
                    c'32
                    [
                    \set stemLeftBeamCount = 3
                    \set stemRightBeamCount = 2
                    d'32
                    \set stemLeftBeamCount = 2
                    \set stemRightBeamCount = 3
                    e'32
                    \set stemLeftBeamCount = 3
                    \set stemRightBeamCount = 0
                    f'32
                    ]
                }

            >>> beam.span_beam_count
            2

        ..  container:: example

            Creates no span beam between adjacent groups in spanner:

            >>> staff = abjad.Staff("c'32 d'32 e'32 f'32")
            >>> durations = [(1, 16), (1, 16)]
            >>> beam = abjad.Beam(
            ...     durations=durations,
            ...     span_beam_count=0,
            ...     )
            >>> abjad.attach(beam, staff[:])
            >>> abjad.show(staff) # doctest: +SKIP

            ..  docs::

                >>> abjad.f(staff)
                \new Staff
                {
                    \set stemLeftBeamCount = 0
                    \set stemRightBeamCount = 3
                    c'32
                    [
                    \set stemLeftBeamCount = 3
                    \set stemRightBeamCount = 0
                    d'32
                    \set stemLeftBeamCount = 0
                    \set stemRightBeamCount = 3
                    e'32
                    \set stemLeftBeamCount = 3
                    \set stemRightBeamCount = 0
                    f'32
                    ]
                }

            >>> beam.span_beam_count
            0

        """
        return self._span_beam_count

    @property
    def stemlet_length(self) -> typing.Optional[typings.Number]:
        r"""
        Gets stemlet length.

        ..  container:: example

            >>> staff = abjad.Staff(
            ...     "r8 c' r c' g'2",
            ...     lilypond_type='RhythmicStaff',
            ...     )
            >>> abjad.setting(staff).auto_beaming = False
            >>> beam = abjad.Beam(beam_rests=True, stemlet_length=2)
            >>> abjad.attach(beam, staff[:-1])
            >>> abjad.show(staff) # doctest: +SKIP

            ..  docs::

                >>> abjad.f(staff)
                \new RhythmicStaff
                \with
                {
                    autoBeaming = ##f
                }
                {
                    \override RhythmicStaff.Stem.stemlet-length = 2
                    r8
                    [
                    c'8
                    r8
                    \revert RhythmicStaff.Stem.stemlet-length
                    c'8
                    ]
                    g'2
                }

        """
        return self._stemlet_length



import numbers
from abjad import system


class LilyPondDimension(system.AbjadObject):
    r"""
    A LilyPond file ``\paper`` block dimension.

    ..  container:: example

        >>> abjad.LilyPondDimension(2, 'in')
        LilyPondDimension(value=2, unit='in')

    Use for LilyPond file ``\paper`` block attributes.
    """

    ### CLASS VARIABLES ###

    __slots__ = (
        '_unit',
        '_value',
        )

    ### INITIALIZER ###

    def __init__(self, value=0, unit='cm'):
        assert isinstance(value, numbers.Number) and 0 <= value
        assert unit in ('cm', 'in', 'mm', 'pt')
        self._value = value
        self._unit = unit

    ### SPECIAL METHODS ###

    def __format__(self, format_specification=''):
        r"""
        Formats LilyPond dimension.

        ..  container:: example

            >>> dimension = abjad.LilyPondDimension(2, 'in')
            >>> print(format(dimension))
            2\in

        Returns string.
        """
        import abjad
        if format_specification in ('', 'lilypond'):
            return self._get_lilypond_format()
        elif format_specification == 'storage':
            return abjad.StorageFormatManager(self).get_storage_format()
        return str(self)

    ### PRIVATE METHODS ###

    def _get_format_pieces(self):
        return [r'{}\{}'.format(self.value, self.unit)]

    def _get_lilypond_format(self):
        return '\n'.join(self._get_format_pieces())

    ### PUBLIC PROPERTIES ###

    @property
    def unit(self):
        """
        Gets unit of LilyPond dimension.

        ..  container:: example

            >>> dimension = abjad.LilyPondDimension(2, 'in')
            >>> dimension.unit
            'in'

        Returns ``'cm'``, ``'in'``, ``'mm'`` or ``'pt'``.
        """
        return self._unit

    @property
    def value(self):
        """
        Gets value of LilyPond dimension.

        ..  container:: example

            >>> dimension = abjad.LilyPondDimension(2, 'in')
            >>> dimension.value
            2

        Returns number.
        """
        return self._value



import typing
from abjad import enums
from abjad import markups
from abjad import typings
from abjad.lilypondnames.LilyPondGrobOverride import LilyPondGrobOverride
from abjad.lilypondnames.LilyPondTweakManager import LilyPondTweakManager
from abjad.system.AbjadValueObject import AbjadValueObject
from abjad.system.LilyPondFormatBundle import LilyPondFormatBundle
from abjad.utilities.String import String
from .LilyPondLiteral import LilyPondLiteral


class StartTextSpan(AbjadValueObject):
    r"""
    LilyPond ``\startTextSpan`` command.

    ..  container:: example

        >>> staff = abjad.Staff("c'4 d' e' f'")
        >>> start_text_span = abjad.StartTextSpan(
        ...     left_text=abjad.Markup('pont.').upright(),
        ...     right_text=abjad.Markup('tasto').upright(),
        ...     style='solid_line_with_arrow',
        ...     )
        >>> abjad.tweak(start_text_span).staff_padding = 2.5
        >>> abjad.attach(start_text_span, staff[0])
        >>> stop_text_span = abjad.StopTextSpan()
        >>> abjad.attach(stop_text_span, staff[-1])
        >>> abjad.show(staff) # doctest: +SKIP

        ..  docs::

            >>> abjad.f(staff)
            \new Staff
            {
                c'4
                - \abjad_solid_line_with_arrow
                - \tweak bound-details.left.text \markup {
                    \concat
                        {
                            \upright
                                pont.
                            \hspace
                                #0.5
                        }
                    }
                - \tweak bound-details.right.text \markup {
                    \upright
                        tasto
                    }
                - \tweak staff-padding #2.5
                \startTextSpan
                d'4
                e'4
                f'4
                \stopTextSpan
            }

    ..  container:: example

        >>> abjad.StartTextSpan()
        StartTextSpan(command='\\startTextSpan', concat_hspace_left=0.5)

    """

    ### CLASS VARIABLES ###

    __slots__ = (
        '_command',
        '_concat_hspace_left',
        '_concat_hspace_right',
        '_direction',
        '_left_broken_text',
        '_left_text',
        '_right_padding',
        '_right_text',
        '_style',
        '_tweaks',
        )

    _publish_storage_format = True

    _styles = (
        'dashed_line_with_arrow',
        'dashed_line_with_hook',
        'invisible_line',
        'solid_line_with_arrow',
        'solid_line_with_hook',
        )

    ### INITIALIZER ###

    def __init__(
        self,
        *,
        command: str = r'\startTextSpan',
        concat_hspace_left: typings.Number = 0.5,
        concat_hspace_right: typings.Number = None,
        direction: enums.VerticalAlignment = None,
        left_broken_text: typing.Union[
            bool, LilyPondLiteral, markups.Markup] = None,
        left_text: typing.Union[LilyPondLiteral, markups.Markup] = None,
        right_padding: typings.Number = None,
        right_text: typing.Union[LilyPondLiteral, markups.Markup] = None,
        style: str = None,
        tweaks: typing.Union[
            typing.List[typing.Tuple], LilyPondTweakManager] = None,
        ) -> None:
        assert isinstance(command, str), repr(command)
        assert command.startswith('\\'), repr(command)
        self._command = command
        if concat_hspace_left is not None:
            assert isinstance(concat_hspace_left, (int, float))
        self._concat_hspace_left = concat_hspace_left
        if concat_hspace_right is not None:
            assert isinstance(concat_hspace_right, (int, float))
        self._concat_hspace_right = concat_hspace_right
        direction_ = String.to_tridirectional_lilypond_symbol(direction)
        self._direction = direction_
        if left_broken_text is not None:
            assert isinstance(left_broken_text, (bool, markups.Markup))
        self._left_broken_text = left_broken_text
        if left_text is not None:
            prototype = (LilyPondLiteral, markups.Markup)
            assert isinstance(left_text, prototype), repr(left_text)
        self._left_text = left_text
        if right_padding is not None:
            assert isinstance(right_padding, (int, float)), repr(right_padding)
        self._right_padding = right_padding
        if right_text is not None:
            prototype = (LilyPondLiteral, markups.Markup)
            assert isinstance(right_text, prototype), repr(right_text)
        self._right_text = right_text
        if style is not None:
            assert style in self._styles, repr(style)
        self._style = style
        self._tweaks = None
        LilyPondTweakManager.set_tweaks(self, tweaks)

    ### PRIVATE METHODS ###

    def _add_direction(self, string):
        if getattr(self, 'direction', False):
            string = f'{self.direction} {string}'
        return string

    def _get_left_broken_text_tweak(self):
        override = LilyPondGrobOverride(
            grob_name='TextSpanner',
            property_path=(
                'bound-details',
                'left-broken',
                'text',
                ),
            value=self.left_broken_text,
            )
        string = override.tweak_string(self)
        return string

    def _get_left_text_tweak(self):
        import abjad
        if isinstance(self.left_text, LilyPondLiteral):
            markup = self.left_text
        else:
            concat_hspace_left_markup = markups.Markup.hspace(
                self.concat_hspace_left)
            markup_list = [self.left_text, concat_hspace_left_markup]
            markup = markups.Markup.concat(markup_list)
        override = LilyPondGrobOverride(
            grob_name='TextSpanner',
            property_path=(
                'bound-details',
                'left',
                'text',
                ),
            value=markup,
            )
        string = override.tweak_string()
        return string

    def _get_lilypond_format_bundle(self, component=None):
        bundle = LilyPondFormatBundle()
        if self.style is not None:
            string = rf'- \abjad_{self.style}'
            bundle.after.spanner_starts.append(string)
        if self.left_text:
            string = self._get_left_text_tweak()
            bundle.after.spanner_starts.append(string)
        if self.left_broken_text is not None:
            string = self._get_left_broken_text_tweak()
            bundle.after.spanner_starts.append(string)
        if self.right_text:
            string = self._get_right_text_tweak()
            bundle.after.spanner_starts.append(string)
        if self.right_padding:
            string = self._get_right_padding_tweak()
            bundle.after.spanner_starts.append(string)
        if self.tweaks:
            tweaks = self.tweaks._list_format_contributions()
            bundle.after.spanner_starts.extend(tweaks)
        string = self._add_direction(self.command)
        bundle.after.spanner_starts.append(string)
        return bundle

    def _get_right_padding_tweak(self):
        override = LilyPondGrobOverride(
            grob_name='TextSpanner',
            property_path=(
                'bound-details',
                'right',
                'padding',
                ),
            value=self.right_padding,
            )
        string = override.tweak_string()
        return string

    def _get_right_text_tweak(self):
        if self.concat_hspace_right is not None:
            number = self.concat_hspace_right
            concat_hspace_right_markup = markups.Markup.hspace(number)
            markup_list = [concat_hspace_right_markup, self.right_text]
            markup = markups.Markup.concat(markup_list)
        else:
            markup = self.right_text
        override = LilyPondGrobOverride(
            grob_name='TextSpanner',
            property_path=(
                'bound-details',
                'right',
                'text',
                ),
            value=markup,
            )
        string = override.tweak_string()
        return string

    def _get_start_command(self):
        string = self._get_parameter_name()
        return rf'\start{string}'

    ### PUBLIC PROPERTIES ###

    @property
    def command(self) -> str:
        r"""
        Gets comand.

        ..  container:: example

            >>> staff = abjad.Staff("c'4 d' e' f'")

            >>> start_text_span = abjad.StartTextSpan(
            ...     left_text=abjad.Markup('pont.').upright(),
            ...     right_text=abjad.Markup('tasto').upright(),
            ...     style='dashed_line_with_arrow',
            ...     )
            >>> abjad.tweak(start_text_span).color = 'blue'
            >>> abjad.tweak(start_text_span).staff_padding = 2.5
            >>> abjad.attach(start_text_span, staff[0])
            >>> stop_text_span = abjad.StopTextSpan()
            >>> abjad.attach(stop_text_span, staff[-1])

            >>> start_text_span = abjad.StartTextSpan(
            ...     command=r'\startTextSpanOne',
            ...     left_text=abjad.Markup('A').upright(),
            ...     right_text=abjad.Markup('B').upright(),
            ...     style='dashed_line_with_arrow',
            ...     )
            >>> abjad.tweak(start_text_span).color = 'red'
            >>> abjad.tweak(start_text_span).staff_padding = 6
            >>> abjad.attach(start_text_span, staff[0])
            >>> stop_text_span = abjad.StopTextSpan(command=r'\stopTextSpanOne')
            >>> abjad.attach(stop_text_span, staff[-1])

            >>> markup = abjad.Markup('SPACER', direction=abjad.Up)
            >>> abjad.tweak(markup).transparent = True
            >>> abjad.attach(markup, staff[0])
            >>> abjad.show(staff) # doctest: +SKIP

            ..  docs::

                >>> abjad.f(staff)
                \new Staff
                {
                    c'4
                    - \tweak transparent ##t
                    ^ \markup { SPACER }
                    - \abjad_dashed_line_with_arrow
                    - \tweak bound-details.left.text \markup {
                        \concat
                            {
                                \upright
                                    pont.
                                \hspace
                                    #0.5
                            }
                        }
                    - \tweak bound-details.right.text \markup {
                        \upright
                            tasto
                        }
                    - \tweak color #blue
                    - \tweak staff-padding #2.5
                    \startTextSpan
                    - \abjad_dashed_line_with_arrow
                    - \tweak bound-details.left.text \markup {
                        \concat
                            {
                                \upright
                                    A
                                \hspace
                                    #0.5
                            }
                        }
                    - \tweak bound-details.right.text \markup {
                        \upright
                            B
                        }
                    - \tweak color #red
                    - \tweak staff-padding #6
                    \startTextSpanOne
                    d'4
                    e'4
                    f'4
                    \stopTextSpan
                    \stopTextSpanOne
                }

            (Spacer text included to prevent docs from clipping output.)

        """
        return self._command

    @property
    def concat_hspace_left(self) -> typings.Number:
        """
        Gets left hspace.

        Only included in LilyPond output when left text is set.
        """
        return self._concat_hspace_left

    @property
    def concat_hspace_right(self) -> typing.Optional[typings.Number]:
        """
        Gets right hspace.

        Only included in LilyPond output when right text is set.
        """
        return self._concat_hspace_right

    @property
    def direction(self) -> typing.Optional[String]:
        """
        Gets direction.
        """
        return self._direction

    @property
    def enchained(self) -> bool:
        """
        Is true.
        """
        return True

    @property
    def left_broken_text(self) -> typing.Union[bool, markups.Markup, None]:
        """
        Gets left broken text.
        """
        return self._left_broken_text

    @property
    def left_text(self) -> typing.Union[LilyPondLiteral, markups.Markup, None]:
        r"""
        Gets left text.

        ..  container:: example

            LilyPond literals are allowed in place of markup:

            >>> staff = abjad.Staff("c'4 d' e' f'")
            >>> left_string = r'\markup \concat { \upright pont. \hspace #0.5 }'
            >>> right_string = r'\markup \upright tasto'
            >>> start_text_span = abjad.StartTextSpan(
            ...     left_text=abjad.LilyPondLiteral(left_string),
            ...     right_text=abjad.LilyPondLiteral(right_string),
            ...     style='solid_line_with_arrow',
            ...     )
            >>> abjad.tweak(start_text_span).staff_padding = 2.5
            >>> abjad.attach(start_text_span, staff[0])
            >>> stop_text_span = abjad.StopTextSpan()
            >>> abjad.attach(stop_text_span, staff[-1])
            >>> abjad.show(staff) # doctest: +SKIP

            ..  docs::

                >>> abjad.f(staff)
                \new Staff
                {
                    c'4
                    - \abjad_solid_line_with_arrow
                    - \tweak bound-details.left.text \markup \concat { \upright pont. \hspace #0.5 }
                    - \tweak bound-details.right.text \markup \upright tasto
                    - \tweak staff-padding #2.5
                    \startTextSpan
                    d'4
                    e'4
                    f'4
                    \stopTextSpan
                }

            Use with custom markup functions.

        """
        return self._left_text

    @property
    def right_padding(self) -> typing.Optional[typings.Number]:
        """
        Gets right padding.
        """
        return self._right_padding

    @property
    def right_text(self) -> typing.Union[
        LilyPondLiteral, markups.Markup, None]:
        """
        Gets right text.
        """
        return self._right_text

    @property
    def spanner_start(self) -> bool:
        """
        Is true.

        ..  container:: example

            >>> abjad.StartTextSpan().spanner_start
            True

        """
        return True

    @property
    def style(self) -> typing.Optional[str]:
        r"""
        Gets style.

        ..  container:: example

            >>> staff = abjad.Staff("c'4 d' e' fs'")
            >>> start_text_span = abjad.StartTextSpan(
            ...     left_text=abjad.Markup('pont.').upright(),
            ...     right_text=abjad.Markup('tasto').upright(),
            ...     style='dashed_line_with_arrow',
            ...     )
            >>> abjad.tweak(start_text_span).staff_padding = 2.5
            >>> abjad.attach(start_text_span, staff[0])
            >>> stop_text_span = abjad.StopTextSpan()
            >>> abjad.attach(stop_text_span, staff[-1])
            >>> abjad.show(staff) # doctest: +SKIP

            ..  docs::

                >>> abjad.f(staff)
                \new Staff
                {
                    c'4
                    - \abjad_dashed_line_with_arrow
                    - \tweak bound-details.left.text \markup {
                        \concat
                            {
                                \upright
                                    pont.
                                \hspace
                                    #0.5
                            }
                        }
                    - \tweak bound-details.right.text \markup {
                        \upright
                            tasto
                        }
                    - \tweak staff-padding #2.5
                    \startTextSpan
                    d'4
                    e'4
                    fs'4
                    \stopTextSpan
                }

        ..  container:: example

            >>> staff = abjad.Staff("c'4 d' e' f'")
            >>> start_text_span = abjad.StartTextSpan(
            ...     left_text=abjad.Markup('pont.').upright(),
            ...     style='dashed_line_with_hook',
            ...     )
            >>> abjad.tweak(start_text_span).staff_padding = 2.5
            >>> abjad.attach(start_text_span, staff[0])
            >>> stop_text_span = abjad.StopTextSpan()
            >>> abjad.attach(stop_text_span, staff[-1])
            >>> abjad.show(staff) # doctest: +SKIP

            ..  docs::

                >>> abjad.f(staff)
                \new Staff
                {
                    c'4
                    - \abjad_dashed_line_with_hook
                    - \tweak bound-details.left.text \markup {
                        \concat
                            {
                                \upright
                                    pont.
                                \hspace
                                    #0.5
                            }
                        }
                    - \tweak staff-padding #2.5
                    \startTextSpan
                    d'4
                    e'4
                    f'4
                    \stopTextSpan
                }

        ..  container:: example

            >>> staff = abjad.Staff("c'4 d' e' f'")
            >>> start_text_span = abjad.StartTextSpan(
            ...     left_text=abjad.Markup('pont.').upright(),
            ...     right_text=abjad.Markup('tasto').upright(),
            ...     style='invisible_line',
            ...     )
            >>> abjad.tweak(start_text_span).staff_padding = 2.5
            >>> abjad.attach(start_text_span, staff[0])
            >>> stop_text_span = abjad.StopTextSpan()
            >>> abjad.attach(stop_text_span, staff[-1])
            >>> abjad.show(staff) # doctest: +SKIP

            ..  docs::

                >>> abjad.f(staff)
                \new Staff
                {
                    c'4
                    - \abjad_invisible_line
                    - \tweak bound-details.left.text \markup {
                        \concat
                            {
                                \upright
                                    pont.
                                \hspace
                                    #0.5
                            }
                        }
                    - \tweak bound-details.right.text \markup {
                        \upright
                            tasto
                        }
                    - \tweak staff-padding #2.5
                    \startTextSpan
                    d'4
                    e'4
                    f'4
                    \stopTextSpan
                }

        ..  container:: example

            >>> staff = abjad.Staff("c'4 d' e' f'")
            >>> start_text_span = abjad.StartTextSpan(
            ...     left_text=abjad.Markup('pont.').upright(),
            ...     right_text=abjad.Markup('tasto').upright(),
            ...     style='solid_line_with_arrow',
            ...     )
            >>> abjad.tweak(start_text_span).staff_padding = 2.5
            >>> abjad.attach(start_text_span, staff[0])
            >>> stop_text_span = abjad.StopTextSpan()
            >>> abjad.attach(stop_text_span, staff[-1])
            >>> abjad.show(staff) # doctest: +SKIP

            ..  docs::

                >>> abjad.f(staff)
                \new Staff
                {
                    c'4
                    - \abjad_solid_line_with_arrow
                    - \tweak bound-details.left.text \markup {
                        \concat
                            {
                                \upright
                                    pont.
                                \hspace
                                    #0.5
                            }
                        }
                    - \tweak bound-details.right.text \markup {
                        \upright
                            tasto
                        }
                    - \tweak staff-padding #2.5
                    \startTextSpan
                    d'4
                    e'4
                    f'4
                    \stopTextSpan
                }

        ..  container:: example

            >>> staff = abjad.Staff("c'4 d' e' f'")
            >>> start_text_span = abjad.StartTextSpan(
            ...     left_text=abjad.Markup('pont.').upright(),
            ...     style='solid_line_with_hook',
            ...     )
            >>> abjad.tweak(start_text_span).staff_padding = 2.5
            >>> abjad.attach(start_text_span, staff[0])
            >>> stop_text_span = abjad.StopTextSpan()
            >>> abjad.attach(stop_text_span, staff[-1])
            >>> abjad.show(staff) # doctest: +SKIP

            ..  docs::

                >>> abjad.f(staff)
                \new Staff
                {
                    c'4
                    - \abjad_solid_line_with_hook
                    - \tweak bound-details.left.text \markup {
                        \concat
                            {
                                \upright
                                    pont.
                                \hspace
                                    #0.5
                            }
                        }
                    - \tweak staff-padding #2.5
                    \startTextSpan
                    d'4
                    e'4
                    f'4
                    \stopTextSpan
                }

        Constrained to ``'dashed_line_with_arrow'``,
        ``'dashed_line_with_hook'``, ``'invisible_line'``,
        ``'solid_line_with_arrow'``, ``'solid_line_with_hook'``, none.
        """
        return self._style

    @property
    def tweaks(self) -> typing.Optional[LilyPondTweakManager]:
        r"""
        Gets tweaks

        ..  container:: example

            >>> staff = abjad.Staff("c'4 d' e' f'")
            >>> start_text_span = abjad.StartTextSpan(
            ...     left_text=abjad.Markup('pont.').upright(),
            ...     right_text=abjad.Markup('tasto').upright(),
            ...     style='dashed_line_with_arrow',
            ...     )
            >>> abjad.tweak(start_text_span).color = 'blue'
            >>> abjad.tweak(start_text_span).staff_padding = 2.5
            >>> abjad.attach(start_text_span, staff[0])
            >>> stop_text_span = abjad.StopTextSpan()
            >>> abjad.attach(stop_text_span, staff[-1])
            >>> abjad.show(staff) # doctest: +SKIP

            ..  docs::

                >>> abjad.f(staff)
                \new Staff
                {
                    c'4
                    - \abjad_dashed_line_with_arrow
                    - \tweak bound-details.left.text \markup {
                        \concat
                            {
                                \upright
                                    pont.
                                \hspace
                                    #0.5
                            }
                        }
                    - \tweak bound-details.right.text \markup {
                        \upright
                            tasto
                        }
                    - \tweak color #blue
                    - \tweak staff-padding #2.5
                    \startTextSpan
                    d'4
                    e'4
                    f'4
                    \stopTextSpan
                }

        ..  container:: example

            REGRESSION. Tweaks survive copy:

            >>> import copy
            >>> start_text_span = abjad.StartTextSpan(
            ...     style='dashed_line_with_arrow',
            ...     )
            >>> abjad.tweak(start_text_span).color = 'blue'
            >>> abjad.tweak(start_text_span).staff_padding = 2.5
            >>> abjad.f(start_text_span)
            abjad.StartTextSpan(
                command='\\startTextSpan',
                concat_hspace_left=0.5,
                style='dashed_line_with_arrow',
                tweaks=LilyPondTweakManager(('color', 'blue'), ('staff_padding', 2.5)),
                )

            >>> start_text_span_2 = copy.copy(start_text_span)
            >>> abjad.f(start_text_span_2)
            abjad.StartTextSpan(
                command='\\startTextSpan',
                concat_hspace_left=0.5,
                style='dashed_line_with_arrow',
                tweaks=LilyPondTweakManager(('color', 'blue'), ('staff_padding', 2.5)),
                )

        """
        return self._tweaks



def all_are_positive_integers(argument):
    """
    Is true when ``argument`` is an iterable collection of positive integers.

    ..  container:: example

        >>> abjad.mathtools.all_are_positive_integers([1, 2, 3, 99])
        True

        >>> abjad.mathtools.all_are_positive_integers(17)
        False

    Returns true or false.
    """
    from abjad import mathtools
    try:
        return all(mathtools.is_positive_integer(_) for _ in argument)
    except TypeError:
        return False



from .Container import Container


class Cluster(Container):
    """
    Cluster.

    ..  container:: example

        >>> cluster = abjad.Cluster("c'8 <d' g'>8 b'8")
        >>> abjad.show(cluster) # doctest: +SKIP

        ..  docs::

            >>> abjad.f(cluster)
            \makeClusters {
                c'8
                <d' g'>8
                b'8
            }

        >>> cluster
        Cluster("c'8 <d' g'>8 b'8")

    """

    ### CLASS VARIABLES ###

    __documentation_section__ = 'Containers'

    __slots__ = ()

    ### PRIVATE METHODS ###

    def _format_open_brackets_slot(self, bundle):
        result = []
        contributor = ('self_brackets', 'open')
        if self.is_simultaneous:
            brackets_open = ['<<']
        else:
            brackets_open = ['{']
        contributions = [r'\makeClusters {}'.format(brackets_open[0])]
        result.append([contributor, contributions])
        return tuple(result)

    def _get_lilypond_format(self):
        self._update_now(indicators=True)
        return self._format_component()



from abjad.utilities.Enumerator import Enumerator
from .Vector import Vector


class IntervalClassVector(Vector):
    """
    Interval-class vector.

    ..  container:: example

        An interval-class vector:

        >>> pitch_segment = abjad.PitchSegment(
        ...     items=[0, 11, 7, 4, 2, 9, 3, 8, 10, 1, 5, 6],
        ...     )
        >>> numbered_interval_class_vector = abjad.IntervalClassVector(
        ...     items=pitch_segment,
        ...     item_class=abjad.NumberedInversionEquivalentIntervalClass,
        ...     )

        >>> items = sorted(numbered_interval_class_vector.items())
        >>> for interval, count in items:
        ...     print(interval, count)
        ...
        1 12
        2 12
        3 12
        4 12
        5 12
        6 6

    """

    ### CLASS VARIABLES ###

    __slots__ = ()

    ### INITIALIZER ###

    def __init__(self, items=None, item_class=None):
        import abjad
        prototype = (
            abjad.PitchSegment,
            abjad.PitchSet,
            abjad.PitchClassSegment,
            abjad.PitchClassSet,
            )
        if isinstance(items, prototype):
            intervals = []
            items = tuple(items)
            enumerator = Enumerator(items)
            pairs = enumerator.yield_pairs()
            for first, second in pairs:
                intervals.append(second - first)
            items = intervals
        Vector.__init__(
            self,
            items=items,
            item_class=item_class,
            )

    ### SPECIAL METHODS ###

    def __repr__(self):
        """
        Gets interpreter representation of interval-class vector.

        ..  container:: example

            Gets interpreter representation of interval-class vector:

            >>> pitch_segment = abjad.PitchSegment(
            ...     items=[0, 11, 7, 4, 2, 9, 3, 8, 10, 1, 5, 6],
            ...     )
            >>> vector = abjad.IntervalClassVector(
            ...     items=pitch_segment,
            ...     item_class=abjad.NumberedInversionEquivalentIntervalClass,
            ...     )

            >>> vector
            IntervalClassVector({1: 12, 2: 12, 3: 12, 4: 12, 5: 12, 6: 6}, item_class=NumberedInversionEquivalentIntervalClass)

        ..  container:: example

            Initializes from interpreter representation of interval-class
            vector:

            >>> abjad.IntervalClassVector(vector)
            IntervalClassVector({1: 12, 2: 12, 3: 12, 4: 12, 5: 12, 6: 6}, item_class=NumberedInversionEquivalentIntervalClass)

        Returns string.
        """
        return super().__repr__()

    ### PRIVATE PROPERTIES ###

    @property
    def _label(self):
        counts = []
        for i in range(7):
            counts.append(self[i])
        counts = ''.join([str(x) for x in counts])
        if len(self) == 13:
            quartertones = []
            for i in range(6):
                quartertones.append(self[i + 0.5])
            quartertones = ''.join([str(x) for x in quartertones])
            return r'\tiny \column { "%s" "%s" }' % (counts, quartertones)
        else:
            return r'\tiny %s' % counts

    @property
    def _named_item_class(self):
        import abjad
        return abjad.NamedIntervalClass

    @property
    def _numbered_item_class(self):
        import abjad
        return abjad.NumberedIntervalClass

    @property
    def _parent_item_class(self):
        import abjad
        return abjad.IntervalClass

    ### PUBLIC METHODS ###

    @classmethod
    def from_selection(
        class_,
        selection,
        item_class=None,
        ):
        """
        Makes interval-class vector from `selection`.

        ..  container:: example

            Makes numbered inversion-equivalent interval-class vector from
            selection:

            >>> vector = abjad.IntervalClassVector.from_selection(
            ...     abjad.Chord("<c' d' b''>4"),
            ...     item_class=abjad.NumberedInversionEquivalentIntervalClass,
            ...     )
            >>> vector
            IntervalClassVector({1: 1, 2: 1, 3: 1}, item_class=NumberedInversionEquivalentIntervalClass)

        ..  container:: example

            Makes numbered interval-class vector from selection:

            >>> vector = abjad.IntervalClassVector.from_selection(
            ...     abjad.Chord("<c' d' b''>4"),
            ...     item_class=abjad.NumberedIntervalClass,
            ...     )
            >>> vector
            IntervalClassVector({-11: 1, -9: 1, -2: 1}, item_class=NumberedIntervalClass)

            .. todo:: This should probabaly be checked. Resulting values
                should probabaly be positive (or signless) instead of negative.

        ..  container:: example

            Makes named interval-class vector from selection:

            >>> vector = abjad.IntervalClassVector.from_selection(
            ...     abjad.Chord("<c' d' b''>4"),
            ...     item_class=None,
            ...     )
            >>> vector
            IntervalClassVector({'-M2': 1, '-M6': 1, '-M7': 1}, item_class=NamedIntervalClass)

            .. todo:: This should probabaly be checked. Resulting values
                should probabaly be positive (or signless) instead of negative.

        Returns new interval-class vector.
        """
        import abjad
        pitch_segment = abjad.PitchSegment.from_selection(selection)
        return class_(
            pitch_segment,
            item_class=item_class,
            )



from abjad.pitch.NamedPitchClass import NamedPitchClass


lilypond_version = "2.19.24"

language_pitch_names = {
    'nederlands':  {
        'ceses': NamedPitchClass('cff'),
        'ceh': NamedPitchClass('cqf'),
        'ces': NamedPitchClass('cf'),
        'ceseh': NamedPitchClass('ctqf'),
        'c': NamedPitchClass('c'),
        'cis': NamedPitchClass('cs'),
        'cih': NamedPitchClass('cqs'),
        'cisih': NamedPitchClass('ctqs'),
        'cisis': NamedPitchClass('css'),
        'deses': NamedPitchClass('dff'),
        'deh': NamedPitchClass('dqf'),
        'des': NamedPitchClass('df'),
        'deseh': NamedPitchClass('dtqf'),
        'd': NamedPitchClass('d'),
        'dis': NamedPitchClass('ds'),
        'dih': NamedPitchClass('dqs'),
        'disih': NamedPitchClass('dtqs'),
        'disis': NamedPitchClass('dss'),
        'eeses': NamedPitchClass('eff'),
        'eses': NamedPitchClass('eff'),
        'eeh': NamedPitchClass('eqf'),
        'ees': NamedPitchClass('ef'),
        'eeseh': NamedPitchClass('etqf'),
        'es': NamedPitchClass('ef'),
        'e': NamedPitchClass('e'),
        'eis': NamedPitchClass('es'),
        'eih': NamedPitchClass('eqs'),
        'eisih': NamedPitchClass('etqs'),
        'eisis': NamedPitchClass('ess'),
        'feses': NamedPitchClass('fff'),
        'feh': NamedPitchClass('fqf'),
        'fes': NamedPitchClass('ff'),
        'feseh': NamedPitchClass('ftqf'),
        'f': NamedPitchClass('f'),
        'fis': NamedPitchClass('fs'),
        'fih': NamedPitchClass('fqs'),
        'fisih': NamedPitchClass('ftqs'),
        'fisis': NamedPitchClass('fss'),
        'geses': NamedPitchClass('gff'),
        'geh': NamedPitchClass('gqf'),
        'ges': NamedPitchClass('gf'),
        'geseh': NamedPitchClass('gtqf'),
        'g': NamedPitchClass('g'),
        'gis': NamedPitchClass('gs'),
        'gih': NamedPitchClass('gqs'),
        'gisih': NamedPitchClass('gtqs'),
        'gisis': NamedPitchClass('gss'),
        'aeses': NamedPitchClass('aff'),
        'ases': NamedPitchClass('aff'),
        'aeh': NamedPitchClass('aqf'),
        'aes': NamedPitchClass('af'),
        'aeseh': NamedPitchClass('atqf'),
        'as': NamedPitchClass('af'),
        'a': NamedPitchClass('a'),
        'ais': NamedPitchClass('as'),
        'aih': NamedPitchClass('aqs'),
        'aisih': NamedPitchClass('atqs'),
        'aisis': NamedPitchClass('ass'),
        'beses': NamedPitchClass('bff'),
        'beh': NamedPitchClass('bqf'),
        'bes': NamedPitchClass('bf'),
        'beseh': NamedPitchClass('btqf'),
        'b': NamedPitchClass('b'),
        'bis': NamedPitchClass('bs'),
        'bih': NamedPitchClass('bqs'),
        'bisih': NamedPitchClass('btqs'),
        'bisis': NamedPitchClass('bss'),
    },

    'catalan':  {
        'dobb': NamedPitchClass('cff'),
        'dob': NamedPitchClass('cf'),
        'do': NamedPitchClass('c'),
        'dod': NamedPitchClass('cs'),
        'dodd': NamedPitchClass('css'),
        'rebb': NamedPitchClass('dff'),
        'reb': NamedPitchClass('df'),
        're': NamedPitchClass('d'),
        'red': NamedPitchClass('ds'),
        'redd': NamedPitchClass('dss'),
        'mibb': NamedPitchClass('eff'),
        'mib': NamedPitchClass('ef'),
        'mi': NamedPitchClass('e'),
        'mid': NamedPitchClass('es'),
        'midd': NamedPitchClass('ess'),
        'fabb': NamedPitchClass('fff'),
        'fab': NamedPitchClass('ff'),
        'fa': NamedPitchClass('f'),
        'fad': NamedPitchClass('fs'),
        'fadd': NamedPitchClass('fss'),
        'solbb': NamedPitchClass('gff'),
        'solb': NamedPitchClass('gf'),
        'sol': NamedPitchClass('g'),
        'sold': NamedPitchClass('gs'),
        'soldd': NamedPitchClass('gss'),
        'labb': NamedPitchClass('aff'),
        'lab': NamedPitchClass('af'),
        'la': NamedPitchClass('a'),
        'lad': NamedPitchClass('as'),
        'ladd': NamedPitchClass('ass'),
        'sibb': NamedPitchClass('bff'),
        'sib': NamedPitchClass('bf'),
        'si': NamedPitchClass('b'),
        'sid': NamedPitchClass('bs'),
        'sidd': NamedPitchClass('bss'),
        'dos': NamedPitchClass('cs'),
        'doss': NamedPitchClass('css'),
        'res': NamedPitchClass('ds'),
        'ress': NamedPitchClass('dss'),
        'mis': NamedPitchClass('es'),
        'miss': NamedPitchClass('ess'),
        'fas': NamedPitchClass('fs'),
        'fass': NamedPitchClass('fss'),
        'sols': NamedPitchClass('gs'),
        'solss': NamedPitchClass('gss'),
        'las': NamedPitchClass('as'),
        'lass': NamedPitchClass('ass'),
        'sis': NamedPitchClass('bs'),
        'siss': NamedPitchClass('bss'),
    },

    'deutsch':  {
        'ceses': NamedPitchClass('cff'),
        'ceseh': NamedPitchClass('ctqf'),
        'ces': NamedPitchClass('cf'),
        'ceh': NamedPitchClass('cqf'),
        'c': NamedPitchClass('c'),
        'cih': NamedPitchClass('cqs'),
        'cis': NamedPitchClass('cs'),
        'cisih': NamedPitchClass('ctqs'),
        'cisis': NamedPitchClass('css'),
        'deses': NamedPitchClass('dff'),
        'deseh': NamedPitchClass('dtqf'),
        'des': NamedPitchClass('df'),
        'deh': NamedPitchClass('dqf'),
        'd': NamedPitchClass('d'),
        'dih': NamedPitchClass('dqs'),
        'dis': NamedPitchClass('ds'),
        'disih': NamedPitchClass('dtqs'),
        'disis': NamedPitchClass('dss'),
        'eses': NamedPitchClass('eff'),
        'eseh': NamedPitchClass('etqf'),
        'es': NamedPitchClass('ef'),
        'eeh': NamedPitchClass('eqf'),
        'e': NamedPitchClass('e'),
        'eih': NamedPitchClass('eqs'),
        'eis': NamedPitchClass('es'),
        'eisih': NamedPitchClass('etqs'),
        'eisis': NamedPitchClass('ess'),
        'feses': NamedPitchClass('fff'),
        'feseh': NamedPitchClass('ftqf'),
        'fes': NamedPitchClass('ff'),
        'feh': NamedPitchClass('fqf'),
        'f': NamedPitchClass('f'),
        'fih': NamedPitchClass('fqs'),
        'fis': NamedPitchClass('fs'),
        'fisih': NamedPitchClass('ftqs'),
        'fisis': NamedPitchClass('fss'),
        'geses': NamedPitchClass('gff'),
        'geseh': NamedPitchClass('gtqf'),
        'ges': NamedPitchClass('gf'),
        'geh': NamedPitchClass('gqf'),
        'g': NamedPitchClass('g'),
        'gih': NamedPitchClass('gqs'),
        'gis': NamedPitchClass('gs'),
        'gisih': NamedPitchClass('gtqs'),
        'gisis': NamedPitchClass('gss'),
        'asas': NamedPitchClass('aff'),
        'asah': NamedPitchClass('atqf'),
        'ases': NamedPitchClass('aff'),
        'aseh': NamedPitchClass('atqf'),
        'as': NamedPitchClass('af'),
        'aeh': NamedPitchClass('aqf'),
        'a': NamedPitchClass('a'),
        'aih': NamedPitchClass('aqs'),
        'ais': NamedPitchClass('as'),
        'aisih': NamedPitchClass('atqs'),
        'aisis': NamedPitchClass('ass'),
        'heses': NamedPitchClass('bff'),
        'heseh': NamedPitchClass('btqf'),
        'b': NamedPitchClass('bf'),
        'beh': NamedPitchClass('bqf'),
        'h': NamedPitchClass('b'),
        'hih': NamedPitchClass('bqs'),
        'his': NamedPitchClass('bs'),
        'hisih': NamedPitchClass('btqs'),
        'hisis': NamedPitchClass('bss'),
    },

    'english':  {
        'cff': NamedPitchClass('cff'),
        'ctqf': NamedPitchClass('ctqf'),
        'cf': NamedPitchClass('cf'),
        'cqf': NamedPitchClass('cqf'),
        'c': NamedPitchClass('c'),
        'cqs': NamedPitchClass('cqs'),
        'cs': NamedPitchClass('cs'),
        'ctqs': NamedPitchClass('ctqs'),
        'css': NamedPitchClass('css'),
        'cx': NamedPitchClass('css'),
        'dff': NamedPitchClass('dff'),
        'dtqf': NamedPitchClass('dtqf'),
        'df': NamedPitchClass('df'),
        'dqf': NamedPitchClass('dqf'),
        'd': NamedPitchClass('d'),
        'dqs': NamedPitchClass('dqs'),
        'ds': NamedPitchClass('ds'),
        'dtqs': NamedPitchClass('dtqs'),
        'dss': NamedPitchClass('dss'),
        'dx': NamedPitchClass('dss'),
        'eff': NamedPitchClass('eff'),
        'etqf': NamedPitchClass('etqf'),
        'ef': NamedPitchClass('ef'),
        'eqf': NamedPitchClass('eqf'),
        'e': NamedPitchClass('e'),
        'eqs': NamedPitchClass('eqs'),
        'es': NamedPitchClass('es'),
        'etqs': NamedPitchClass('etqs'),
        'ess': NamedPitchClass('ess'),
        'ex': NamedPitchClass('ess'),
        'fff': NamedPitchClass('fff'),
        'ftqf': NamedPitchClass('ftqf'),
        'ff': NamedPitchClass('ff'),
        'fqf': NamedPitchClass('fqf'),
        'f': NamedPitchClass('f'),
        'fqs': NamedPitchClass('fqs'),
        'fs': NamedPitchClass('fs'),
        'ftqs': NamedPitchClass('ftqs'),
        'fss': NamedPitchClass('fss'),
        'fx': NamedPitchClass('fss'),
        'gff': NamedPitchClass('gff'),
        'gtqf': NamedPitchClass('gtqf'),
        'gf': NamedPitchClass('gf'),
        'gqf': NamedPitchClass('gqf'),
        'g': NamedPitchClass('g'),
        'gqs': NamedPitchClass('gqs'),
        'gs': NamedPitchClass('gs'),
        'gtqs': NamedPitchClass('gtqs'),
        'gss': NamedPitchClass('gss'),
        'gx': NamedPitchClass('gss'),
        'aff': NamedPitchClass('aff'),
        'atqf': NamedPitchClass('atqf'),
        'af': NamedPitchClass('af'),
        'aqf': NamedPitchClass('aqf'),
        'a': NamedPitchClass('a'),
        'aqs': NamedPitchClass('aqs'),
        'as': NamedPitchClass('as'),
        'atqs': NamedPitchClass('atqs'),
        'ass': NamedPitchClass('ass'),
        'ax': NamedPitchClass('ass'),
        'bff': NamedPitchClass('bff'),
        'btqf': NamedPitchClass('btqf'),
        'bf': NamedPitchClass('bf'),
        'bqf': NamedPitchClass('bqf'),
        'b': NamedPitchClass('b'),
        'bqs': NamedPitchClass('bqs'),
        'bs': NamedPitchClass('bs'),
        'btqs': NamedPitchClass('btqs'),
        'bss': NamedPitchClass('bss'),
        'bx': NamedPitchClass('bss'),
        'c-flatflat': NamedPitchClass('cff'),
        'c-flat': NamedPitchClass('cf'),
        'c-natural': NamedPitchClass('c'),
        'c-sharp': NamedPitchClass('cs'),
        'c-sharpsharp': NamedPitchClass('css'),
        'd-flatflat': NamedPitchClass('dff'),
        'd-flat': NamedPitchClass('df'),
        'd-natural': NamedPitchClass('d'),
        'd-sharp': NamedPitchClass('ds'),
        'd-sharpsharp': NamedPitchClass('dss'),
        'e-flatflat': NamedPitchClass('eff'),
        'e-flat': NamedPitchClass('ef'),
        'e-natural': NamedPitchClass('e'),
        'e-sharp': NamedPitchClass('es'),
        'e-sharpsharp': NamedPitchClass('ess'),
        'f-flatflat': NamedPitchClass('fff'),
        'f-flat': NamedPitchClass('ff'),
        'f-natural': NamedPitchClass('f'),
        'f-sharp': NamedPitchClass('fs'),
        'f-sharpsharp': NamedPitchClass('fss'),
        'g-flatflat': NamedPitchClass('gff'),
        'g-flat': NamedPitchClass('gf'),
        'g-natural': NamedPitchClass('g'),
        'g-sharp': NamedPitchClass('gs'),
        'g-sharpsharp': NamedPitchClass('gss'),
        'a-flatflat': NamedPitchClass('aff'),
        'a-flat': NamedPitchClass('af'),
        'a-natural': NamedPitchClass('a'),
        'a-sharp': NamedPitchClass('as'),
        'a-sharpsharp': NamedPitchClass('ass'),
        'b-flatflat': NamedPitchClass('bff'),
        'b-flat': NamedPitchClass('bf'),
        'b-natural': NamedPitchClass('b'),
        'b-sharp': NamedPitchClass('bs'),
        'b-sharpsharp': NamedPitchClass('bss'),
    },

    'espanol':  {
        'dobb': NamedPitchClass('cff'),
        'dotcb': NamedPitchClass('ctqf'),
        'dob': NamedPitchClass('cf'),
        'docb': NamedPitchClass('cqf'),
        'do': NamedPitchClass('c'),
        'docs': NamedPitchClass('cqs'),
        'dos': NamedPitchClass('cs'),
        'dotcs': NamedPitchClass('ctqs'),
        'doss': NamedPitchClass('css'),
        'dox': NamedPitchClass('css'),
        'rebb': NamedPitchClass('dff'),
        'retcb': NamedPitchClass('dtqf'),
        'reb': NamedPitchClass('df'),
        'recb': NamedPitchClass('dqf'),
        're': NamedPitchClass('d'),
        'recs': NamedPitchClass('dqs'),
        'res': NamedPitchClass('ds'),
        'retcs': NamedPitchClass('dtqs'),
        'ress': NamedPitchClass('dss'),
        'rex': NamedPitchClass('dss'),
        'mibb': NamedPitchClass('eff'),
        'mitcb': NamedPitchClass('etqf'),
        'mib': NamedPitchClass('ef'),
        'micb': NamedPitchClass('eqf'),
        'mi': NamedPitchClass('e'),
        'mics': NamedPitchClass('eqs'),
        'mis': NamedPitchClass('es'),
        'mitcs': NamedPitchClass('etqs'),
        'miss': NamedPitchClass('ess'),
        'mix': NamedPitchClass('ess'),
        'fabb': NamedPitchClass('fff'),
        'fatcb': NamedPitchClass('ftqf'),
        'fab': NamedPitchClass('ff'),
        'facb': NamedPitchClass('fqf'),
        'fa': NamedPitchClass('f'),
        'facs': NamedPitchClass('fqs'),
        'fas': NamedPitchClass('fs'),
        'fatcs': NamedPitchClass('ftqs'),
        'fass': NamedPitchClass('fss'),
        'fax': NamedPitchClass('fss'),
        'solbb': NamedPitchClass('gff'),
        'soltcb': NamedPitchClass('gtqf'),
        'solb': NamedPitchClass('gf'),
        'solcb': NamedPitchClass('gqf'),
        'sol': NamedPitchClass('g'),
        'solcs': NamedPitchClass('gqs'),
        'sols': NamedPitchClass('gs'),
        'soltcs': NamedPitchClass('gtqs'),
        'solss': NamedPitchClass('gss'),
        'solx': NamedPitchClass('gss'),
        'labb': NamedPitchClass('aff'),
        'latcb': NamedPitchClass('atqf'),
        'lab': NamedPitchClass('af'),
        'lacb': NamedPitchClass('aqf'),
        'la': NamedPitchClass('a'),
        'lacs': NamedPitchClass('aqs'),
        'las': NamedPitchClass('as'),
        'latcs': NamedPitchClass('atqs'),
        'lass': NamedPitchClass('ass'),
        'lax': NamedPitchClass('ass'),
        'sibb': NamedPitchClass('bff'),
        'sitcb': NamedPitchClass('btqf'),
        'sib': NamedPitchClass('bf'),
        'sicb': NamedPitchClass('bqf'),
        'si': NamedPitchClass('b'),
        'sics': NamedPitchClass('bqs'),
        'sis': NamedPitchClass('bs'),
        'sitcs': NamedPitchClass('btqs'),
        'siss': NamedPitchClass('bss'),
        'six': NamedPitchClass('bss'),
    },

    'italiano':  {
        'dobb': NamedPitchClass('cff'),
        'dobsb': NamedPitchClass('ctqf'),
        'dob': NamedPitchClass('cf'),
        'dosb': NamedPitchClass('cqf'),
        'do': NamedPitchClass('c'),
        'dosd': NamedPitchClass('cqs'),
        'dod': NamedPitchClass('cs'),
        'dodsd': NamedPitchClass('ctqs'),
        'dodd': NamedPitchClass('css'),
        'rebb': NamedPitchClass('dff'),
        'rebsb': NamedPitchClass('dtqf'),
        'reb': NamedPitchClass('df'),
        'resb': NamedPitchClass('dqf'),
        're': NamedPitchClass('d'),
        'resd': NamedPitchClass('dqs'),
        'red': NamedPitchClass('ds'),
        'redsd': NamedPitchClass('dtqs'),
        'redd': NamedPitchClass('dss'),
        'mibb': NamedPitchClass('eff'),
        'mibsb': NamedPitchClass('etqf'),
        'mib': NamedPitchClass('ef'),
        'misb': NamedPitchClass('eqf'),
        'mi': NamedPitchClass('e'),
        'misd': NamedPitchClass('eqs'),
        'mid': NamedPitchClass('es'),
        'midsd': NamedPitchClass('etqs'),
        'midd': NamedPitchClass('ess'),
        'fabb': NamedPitchClass('fff'),
        'fabsb': NamedPitchClass('ftqf'),
        'fab': NamedPitchClass('ff'),
        'fasb': NamedPitchClass('fqf'),
        'fa': NamedPitchClass('f'),
        'fasd': NamedPitchClass('fqs'),
        'fad': NamedPitchClass('fs'),
        'fadsd': NamedPitchClass('ftqs'),
        'fadd': NamedPitchClass('fss'),
        'solbb': NamedPitchClass('gff'),
        'solbsb': NamedPitchClass('gtqf'),
        'solb': NamedPitchClass('gf'),
        'solsb': NamedPitchClass('gqf'),
        'sol': NamedPitchClass('g'),
        'solsd': NamedPitchClass('gqs'),
        'sold': NamedPitchClass('gs'),
        'soldsd': NamedPitchClass('gtqs'),
        'soldd': NamedPitchClass('gss'),
        'labb': NamedPitchClass('aff'),
        'labsb': NamedPitchClass('atqf'),
        'lab': NamedPitchClass('af'),
        'lasb': NamedPitchClass('aqf'),
        'la': NamedPitchClass('a'),
        'lasd': NamedPitchClass('aqs'),
        'lad': NamedPitchClass('as'),
        'ladsd': NamedPitchClass('atqs'),
        'ladd': NamedPitchClass('ass'),
        'sibb': NamedPitchClass('bff'),
        'sibsb': NamedPitchClass('btqf'),
        'sib': NamedPitchClass('bf'),
        'sisb': NamedPitchClass('bqf'),
        'si': NamedPitchClass('b'),
        'sisd': NamedPitchClass('bqs'),
        'sid': NamedPitchClass('bs'),
        'sidsd': NamedPitchClass('btqs'),
        'sidd': NamedPitchClass('bss'),
    },

    'norsk':  {
        'ceses': NamedPitchClass('cff'),
        'cessess': NamedPitchClass('cff'),
        'ces': NamedPitchClass('cf'),
        'cess': NamedPitchClass('cf'),
        'c': NamedPitchClass('c'),
        'cis': NamedPitchClass('cs'),
        'ciss': NamedPitchClass('cs'),
        'cisis': NamedPitchClass('css'),
        'cississ': NamedPitchClass('css'),
        'deses': NamedPitchClass('dff'),
        'dessess': NamedPitchClass('dff'),
        'des': NamedPitchClass('df'),
        'dess': NamedPitchClass('df'),
        'd': NamedPitchClass('d'),
        'dis': NamedPitchClass('ds'),
        'diss': NamedPitchClass('ds'),
        'disis': NamedPitchClass('dss'),
        'dississ': NamedPitchClass('dss'),
        'eeses': NamedPitchClass('eff'),
        'eessess': NamedPitchClass('eff'),
        'eses': NamedPitchClass('eff'),
        'essess': NamedPitchClass('eff'),
        'ees': NamedPitchClass('ef'),
        'eess': NamedPitchClass('ef'),
        'es': NamedPitchClass('ef'),
        'ess': NamedPitchClass('ef'),
        'e': NamedPitchClass('e'),
        'eis': NamedPitchClass('es'),
        'eiss': NamedPitchClass('es'),
        'eisis': NamedPitchClass('ess'),
        'eississ': NamedPitchClass('ess'),
        'feses': NamedPitchClass('fff'),
        'fessess': NamedPitchClass('fff'),
        'fes': NamedPitchClass('ff'),
        'fess': NamedPitchClass('ff'),
        'f': NamedPitchClass('f'),
        'fis': NamedPitchClass('fs'),
        'fiss': NamedPitchClass('fs'),
        'fisis': NamedPitchClass('fss'),
        'fississ': NamedPitchClass('fss'),
        'geses': NamedPitchClass('gff'),
        'gessess': NamedPitchClass('gff'),
        'ges': NamedPitchClass('gf'),
        'gess': NamedPitchClass('gf'),
        'g': NamedPitchClass('g'),
        'g': NamedPitchClass('g'),
        'gis': NamedPitchClass('gs'),
        'giss': NamedPitchClass('gs'),
        'gisis': NamedPitchClass('gss'),
        'gississ': NamedPitchClass('gss'),
        'aeses': NamedPitchClass('aff'),
        'aessess': NamedPitchClass('aff'),
        'ases': NamedPitchClass('aff'),
        'assess': NamedPitchClass('aff'),
        'aes': NamedPitchClass('af'),
        'aess': NamedPitchClass('af'),
        'as': NamedPitchClass('af'),
        'ass': NamedPitchClass('af'),
        'a': NamedPitchClass('a'),
        'ais': NamedPitchClass('as'),
        'aiss': NamedPitchClass('as'),
        'aisis': NamedPitchClass('ass'),
        'aississ': NamedPitchClass('ass'),
        'bes': NamedPitchClass('bff'),
        'bess': NamedPitchClass('bff'),
        'b': NamedPitchClass('bf'),
        'b': NamedPitchClass('bf'),
        'h': NamedPitchClass('b'),
        'his': NamedPitchClass('bs'),
        'hiss': NamedPitchClass('bs'),
        'hisis': NamedPitchClass('bss'),
        'hississ': NamedPitchClass('bss'),
    },

    'portugues':  {
        'dobb': NamedPitchClass('cff'),
        'dobtqt': NamedPitchClass('ctqf'),
        'dob': NamedPitchClass('cf'),
        'dobqt': NamedPitchClass('cqf'),
        'do': NamedPitchClass('c'),
        'dosqt': NamedPitchClass('cqs'),
        'dos': NamedPitchClass('cs'),
        'dostqt': NamedPitchClass('ctqs'),
        'doss': NamedPitchClass('css'),
        'rebb': NamedPitchClass('dff'),
        'rebtqt': NamedPitchClass('dtqf'),
        'reb': NamedPitchClass('df'),
        'rebqt': NamedPitchClass('dqf'),
        're': NamedPitchClass('d'),
        'resqt': NamedPitchClass('dqs'),
        'res': NamedPitchClass('ds'),
        'restqt': NamedPitchClass('dtqs'),
        'ress': NamedPitchClass('dss'),
        'mibb': NamedPitchClass('eff'),
        'mibtqt': NamedPitchClass('etqf'),
        'mib': NamedPitchClass('ef'),
        'mibqt': NamedPitchClass('eqf'),
        'mi': NamedPitchClass('e'),
        'misqt': NamedPitchClass('eqs'),
        'mis': NamedPitchClass('es'),
        'mistqt': NamedPitchClass('etqs'),
        'miss': NamedPitchClass('ess'),
        'fabb': NamedPitchClass('fff'),
        'fabtqt': NamedPitchClass('ftqf'),
        'fab': NamedPitchClass('ff'),
        'fabqt': NamedPitchClass('fqf'),
        'fa': NamedPitchClass('f'),
        'fasqt': NamedPitchClass('fqs'),
        'fas': NamedPitchClass('fs'),
        'fastqt': NamedPitchClass('ftqs'),
        'fass': NamedPitchClass('fss'),
        'solbb': NamedPitchClass('gff'),
        'solbtqt': NamedPitchClass('gtqf'),
        'solb': NamedPitchClass('gf'),
        'solbqt': NamedPitchClass('gqf'),
        'sol': NamedPitchClass('g'),
        'solsqt': NamedPitchClass('gqs'),
        'sols': NamedPitchClass('gs'),
        'solstqt': NamedPitchClass('gtqs'),
        'solss': NamedPitchClass('gss'),
        'labb': NamedPitchClass('aff'),
        'labtqt': NamedPitchClass('atqf'),
        'lab': NamedPitchClass('af'),
        'labqt': NamedPitchClass('aqf'),
        'la': NamedPitchClass('a'),
        'lasqt': NamedPitchClass('aqs'),
        'las': NamedPitchClass('as'),
        'lastqt': NamedPitchClass('atqs'),
        'lass': NamedPitchClass('ass'),
        'sibb': NamedPitchClass('bff'),
        'sibtqt': NamedPitchClass('btqf'),
        'sib': NamedPitchClass('bf'),
        'sibqt': NamedPitchClass('bqf'),
        'si': NamedPitchClass('b'),
        'sisqt': NamedPitchClass('bqs'),
        'sis': NamedPitchClass('bs'),
        'sistqt': NamedPitchClass('btqs'),
        'siss': NamedPitchClass('bss'),
    },

    'suomi':  {
        'ceses': NamedPitchClass('cff'),
        'ces': NamedPitchClass('cf'),
        'c': NamedPitchClass('c'),
        'cis': NamedPitchClass('cs'),
        'cisis': NamedPitchClass('css'),
        'deses': NamedPitchClass('dff'),
        'des': NamedPitchClass('df'),
        'd': NamedPitchClass('d'),
        'dis': NamedPitchClass('ds'),
        'disis': NamedPitchClass('dss'),
        'eses': NamedPitchClass('eff'),
        'es': NamedPitchClass('ef'),
        'e': NamedPitchClass('e'),
        'eis': NamedPitchClass('es'),
        'eisis': NamedPitchClass('ess'),
        'feses': NamedPitchClass('fff'),
        'fes': NamedPitchClass('ff'),
        'f': NamedPitchClass('f'),
        'fis': NamedPitchClass('fs'),
        'fisis': NamedPitchClass('fss'),
        'geses': NamedPitchClass('gff'),
        'ges': NamedPitchClass('gf'),
        'g': NamedPitchClass('g'),
        'gis': NamedPitchClass('gs'),
        'gisis': NamedPitchClass('gss'),
        'asas': NamedPitchClass('aff'),
        'ases': NamedPitchClass('aff'),
        'as': NamedPitchClass('af'),
        'a': NamedPitchClass('a'),
        'ais': NamedPitchClass('as'),
        'aisis': NamedPitchClass('ass'),
        'bb': NamedPitchClass('bff'),
        'bes': NamedPitchClass('bff'),
        'heses': NamedPitchClass('bff'),
        'b': NamedPitchClass('bf'),
        'h': NamedPitchClass('b'),
        'his': NamedPitchClass('bs'),
        'hisis': NamedPitchClass('bss'),
    },

    'svenska':  {
        'cessess': NamedPitchClass('cff'),
        'cess': NamedPitchClass('cf'),
        'c': NamedPitchClass('c'),
        'ciss': NamedPitchClass('cs'),
        'cississ': NamedPitchClass('css'),
        'dessess': NamedPitchClass('dff'),
        'dess': NamedPitchClass('df'),
        'd': NamedPitchClass('d'),
        'diss': NamedPitchClass('ds'),
        'dississ': NamedPitchClass('dss'),
        'essess': NamedPitchClass('eff'),
        'ess': NamedPitchClass('ef'),
        'e': NamedPitchClass('e'),
        'eiss': NamedPitchClass('es'),
        'eississ': NamedPitchClass('ess'),
        'fessess': NamedPitchClass('fff'),
        'fess': NamedPitchClass('ff'),
        'f': NamedPitchClass('f'),
        'fiss': NamedPitchClass('fs'),
        'fississ': NamedPitchClass('fss'),
        'gessess': NamedPitchClass('gff'),
        'gess': NamedPitchClass('gf'),
        'g': NamedPitchClass('g'),
        'giss': NamedPitchClass('gs'),
        'gississ': NamedPitchClass('gss'),
        'assess': NamedPitchClass('aff'),
        'ass': NamedPitchClass('af'),
        'a': NamedPitchClass('a'),
        'aiss': NamedPitchClass('as'),
        'aississ': NamedPitchClass('ass'),
        'hessess': NamedPitchClass('bff'),
        'b': NamedPitchClass('bf'),
        'h': NamedPitchClass('b'),
        'hiss': NamedPitchClass('bs'),
        'hississ': NamedPitchClass('bss'),
    },

    'vlaams':  {
        'dobb': NamedPitchClass('cff'),
        'dob': NamedPitchClass('cf'),
        'do': NamedPitchClass('c'),
        'dok': NamedPitchClass('cs'),
        'dokk': NamedPitchClass('css'),
        'rebb': NamedPitchClass('dff'),
        'reb': NamedPitchClass('df'),
        're': NamedPitchClass('d'),
        'rek': NamedPitchClass('ds'),
        'rekk': NamedPitchClass('dss'),
        'mibb': NamedPitchClass('eff'),
        'mib': NamedPitchClass('ef'),
        'mi': NamedPitchClass('e'),
        'mik': NamedPitchClass('es'),
        'mikk': NamedPitchClass('ess'),
        'fabb': NamedPitchClass('fff'),
        'fab': NamedPitchClass('ff'),
        'fa': NamedPitchClass('f'),
        'fak': NamedPitchClass('fs'),
        'fakk': NamedPitchClass('fss'),
        'solbb': NamedPitchClass('gff'),
        'solb': NamedPitchClass('gf'),
        'sol': NamedPitchClass('g'),
        'solk': NamedPitchClass('gs'),
        'solkk': NamedPitchClass('gss'),
        'labb': NamedPitchClass('aff'),
        'lab': NamedPitchClass('af'),
        'la': NamedPitchClass('a'),
        'lak': NamedPitchClass('as'),
        'lakk': NamedPitchClass('ass'),
        'sibb': NamedPitchClass('bff'),
        'sib': NamedPitchClass('bf'),
        'si': NamedPitchClass('b'),
        'sik': NamedPitchClass('bs'),
        'sikk': NamedPitchClass('bss'),
    },

    'español':  {
        'dobb': NamedPitchClass('cff'),
        'dotcb': NamedPitchClass('ctqf'),
        'dob': NamedPitchClass('cf'),
        'docb': NamedPitchClass('cqf'),
        'do': NamedPitchClass('c'),
        'docs': NamedPitchClass('cqs'),
        'dos': NamedPitchClass('cs'),
        'dotcs': NamedPitchClass('ctqs'),
        'doss': NamedPitchClass('css'),
        'dox': NamedPitchClass('css'),
        'rebb': NamedPitchClass('dff'),
        'retcb': NamedPitchClass('dtqf'),
        'reb': NamedPitchClass('df'),
        'recb': NamedPitchClass('dqf'),
        're': NamedPitchClass('d'),
        'recs': NamedPitchClass('dqs'),
        'res': NamedPitchClass('ds'),
        'retcs': NamedPitchClass('dtqs'),
        'ress': NamedPitchClass('dss'),
        'rex': NamedPitchClass('dss'),
        'mibb': NamedPitchClass('eff'),
        'mitcb': NamedPitchClass('etqf'),
        'mib': NamedPitchClass('ef'),
        'micb': NamedPitchClass('eqf'),
        'mi': NamedPitchClass('e'),
        'mics': NamedPitchClass('eqs'),
        'mis': NamedPitchClass('es'),
        'mitcs': NamedPitchClass('etqs'),
        'miss': NamedPitchClass('ess'),
        'mix': NamedPitchClass('ess'),
        'fabb': NamedPitchClass('fff'),
        'fatcb': NamedPitchClass('ftqf'),
        'fab': NamedPitchClass('ff'),
        'facb': NamedPitchClass('fqf'),
        'fa': NamedPitchClass('f'),
        'facs': NamedPitchClass('fqs'),
        'fas': NamedPitchClass('fs'),
        'fatcs': NamedPitchClass('ftqs'),
        'fass': NamedPitchClass('fss'),
        'fax': NamedPitchClass('fss'),
        'solbb': NamedPitchClass('gff'),
        'soltcb': NamedPitchClass('gtqf'),
        'solb': NamedPitchClass('gf'),
        'solcb': NamedPitchClass('gqf'),
        'sol': NamedPitchClass('g'),
        'solcs': NamedPitchClass('gqs'),
        'sols': NamedPitchClass('gs'),
        'soltcs': NamedPitchClass('gtqs'),
        'solss': NamedPitchClass('gss'),
        'solx': NamedPitchClass('gss'),
        'labb': NamedPitchClass('aff'),
        'latcb': NamedPitchClass('atqf'),
        'lab': NamedPitchClass('af'),
        'lacb': NamedPitchClass('aqf'),
        'la': NamedPitchClass('a'),
        'lacs': NamedPitchClass('aqs'),
        'las': NamedPitchClass('as'),
        'latcs': NamedPitchClass('atqs'),
        'lass': NamedPitchClass('ass'),
        'lax': NamedPitchClass('ass'),
        'sibb': NamedPitchClass('bff'),
        'sitcb': NamedPitchClass('btqf'),
        'sib': NamedPitchClass('bf'),
        'sicb': NamedPitchClass('bqf'),
        'si': NamedPitchClass('b'),
        'sics': NamedPitchClass('bqs'),
        'sis': NamedPitchClass('bs'),
        'sitcs': NamedPitchClass('btqs'),
        'siss': NamedPitchClass('bss'),
        'six': NamedPitchClass('bss'),
    },

    'français':  {
        'dobb': NamedPitchClass('cff'),
        'dobsb': NamedPitchClass('ctqf'),
        'dob': NamedPitchClass('cf'),
        'dosb': NamedPitchClass('cqf'),
        'do': NamedPitchClass('c'),
        'dosd': NamedPitchClass('cqs'),
        'dod': NamedPitchClass('cs'),
        'dodsd': NamedPitchClass('ctqs'),
        'dodd': NamedPitchClass('css'),
        'rebb': NamedPitchClass('dff'),
        'rebsb': NamedPitchClass('dtqf'),
        'reb': NamedPitchClass('df'),
        'resb': NamedPitchClass('dqf'),
        're': NamedPitchClass('d'),
        'resd': NamedPitchClass('dqs'),
        'red': NamedPitchClass('ds'),
        'redsd': NamedPitchClass('dtqs'),
        'redd': NamedPitchClass('dss'),
        'mibb': NamedPitchClass('eff'),
        'mibsb': NamedPitchClass('etqf'),
        'mib': NamedPitchClass('ef'),
        'misb': NamedPitchClass('eqf'),
        'mi': NamedPitchClass('e'),
        'misd': NamedPitchClass('eqs'),
        'mid': NamedPitchClass('es'),
        'midsd': NamedPitchClass('etqs'),
        'midd': NamedPitchClass('ess'),
        'fabb': NamedPitchClass('fff'),
        'fabsb': NamedPitchClass('ftqf'),
        'fab': NamedPitchClass('ff'),
        'fasb': NamedPitchClass('fqf'),
        'fa': NamedPitchClass('f'),
        'fasd': NamedPitchClass('fqs'),
        'fad': NamedPitchClass('fs'),
        'fadsd': NamedPitchClass('ftqs'),
        'fadd': NamedPitchClass('fss'),
        'solbb': NamedPitchClass('gff'),
        'solbsb': NamedPitchClass('gtqf'),
        'solb': NamedPitchClass('gf'),
        'solsb': NamedPitchClass('gqf'),
        'sol': NamedPitchClass('g'),
        'solsd': NamedPitchClass('gqs'),
        'sold': NamedPitchClass('gs'),
        'soldsd': NamedPitchClass('gtqs'),
        'soldd': NamedPitchClass('gss'),
        'labb': NamedPitchClass('aff'),
        'labsb': NamedPitchClass('atqf'),
        'lab': NamedPitchClass('af'),
        'lasb': NamedPitchClass('aqf'),
        'la': NamedPitchClass('a'),
        'lasd': NamedPitchClass('aqs'),
        'lad': NamedPitchClass('as'),
        'ladsd': NamedPitchClass('atqs'),
        'ladd': NamedPitchClass('ass'),
        'sibb': NamedPitchClass('bff'),
        'sibsb': NamedPitchClass('btqf'),
        'sib': NamedPitchClass('bf'),
        'sisb': NamedPitchClass('bqf'),
        'si': NamedPitchClass('b'),
        'sisd': NamedPitchClass('bqs'),
        'sid': NamedPitchClass('bs'),
        'sidsd': NamedPitchClass('btqs'),
        'sidd': NamedPitchClass('bss'),
    },

    }



def label(client=None, deactivate=None, tag=None):
    r"""
    Makes label agent or label expression.

    ..  container:: example

        Labels logical ties with start offsets:

        >>> staff = abjad.Staff(r"\times 2/3 { c'4 d'4 e'4 ~ } e'4 ef'4")
        >>> abjad.label(staff).with_start_offsets(direction=abjad.Up)
        Duration(1, 1)

        >>> abjad.override(staff).text_script.staff_padding = 4
        >>> abjad.override(staff).tuplet_bracket.staff_padding = 0
        >>> abjad.show(staff) # doctest: +SKIP

        ..  docs::

            >>> abjad.f(staff)
            \new Staff
            \with
            {
                \override TextScript.staff-padding = #4
                \override TupletBracket.staff-padding = #0
            }
            {
                \times 2/3 {
                    c'4
                    ^ \markup { 0 }
                    d'4
                    ^ \markup { 1/6 }
                    e'4
                    ^ \markup { 1/3 }
                    ~
                }
                e'4
                ef'4
                ^ \markup { 3/4 }
            }

        See the ``Label`` API entry for many more examples.

    ..  container:: example expression

        Initializes positionally:

        >>> expression = abjad.label()
        >>> expression(staff)
        Label(client=<Staff{3}>)

        Initializes from keyword:

        >>> expression = abjad.label()
        >>> expression(client=staff)
        Label(client=<Staff{3}>)

        Makes label expression:

            >>> expression = abjad.label()
            >>> expression = expression.with_start_offsets()

        >>> staff = abjad.Staff(r"\times 2/3 { c'4 d'4 e'4 ~ } e'4 ef'4")
        >>> expression(staff)
        Duration(1, 1)

        >>> abjad.override(staff).text_script.staff_padding = 4
        >>> abjad.override(staff).tuplet_bracket.staff_padding = 0
        >>> abjad.show(staff) # doctest: +SKIP

        ..  docs::

            >>> abjad.f(staff)
            \new Staff
            \with
            {
                \override TextScript.staff-padding = #4
                \override TupletBracket.staff-padding = #0
            }
            {
                \times 2/3 {
                    c'4
                    ^ \markup { 0 }
                    d'4
                    ^ \markup { 1/6 }
                    e'4
                    ^ \markup { 1/3 }
                    ~
                }
                e'4
                ef'4
                ^ \markup { 3/4 }
            }

        See the ``Label`` API entry for many more examples.

    Returns label agent when ``client`` is not none.

    Returns label expression when ``client`` is none.
    """
    import abjad
    if client is not None:
        return abjad.Label(
            client=client,
            deactivate=deactivate,
            tag=tag,
            )
    expression = abjad.Expression()
    expression = expression.label(tag=tag)
    return expression



import copy
import typing
from abjad import enums
from abjad.lilypondnames.LilyPondTweakManager import LilyPondTweakManager
from abjad.system.AbjadValueObject import AbjadValueObject
from abjad.system.FormatSpecification import FormatSpecification
from abjad.system.LilyPondFormatBundle import LilyPondFormatBundle
from abjad.system.StorageFormatManager import StorageFormatManager
from abjad.top.attach import attach
from abjad.utilities.String import String


class Articulation(AbjadValueObject):
    r"""
    Articulation.

    ..  container:: example

        Initializes from name:

        >>> abjad.Articulation('staccato')
        Articulation('staccato')

    ..  container:: example

        Initializes from abbreviation:

        >>> abjad.Articulation('.')
        Articulation('.')

    ..  container:: example

        Initializes from other articulation:

        >>> articulation = abjad.Articulation('staccato')
        >>> abjad.Articulation(articulation)
        Articulation('staccato')

    ..  container:: example

        Initializes with direction:

        >>> abjad.Articulation('staccato', direction=abjad.Up)
        Articulation('staccato', Up)

    .. container:: example

        Use ``attach()`` to attach articulations to notes, rests or chords:

        >>> note = abjad.Note("c'4")
        >>> articulation = abjad.Articulation('staccato')
        >>> abjad.attach(articulation, note)
        >>> abjad.show(note) # doctest: +SKIP

    ..  todo:: Simplify initializer. Allow only initialization from name.
        Implement new ``from_abbreviation()`` and ``from_articulation()``
        methods to replace existing initializer polymorphism.

    ..  container:: example

        Works with new:

        >>> abjad.new(abjad.Articulation('.'))
        Articulation('.')

    """

    ### CLASS VARIABLES ###

    __slots__ = (
        '_direction',
        '_format_slot',
        '_name',
        '_tweaks',
        )

    _articulations_supported = (
        'accent',
        'marcato',
        'staccatissimo',
        'espressivo'
        'staccato',
        'tenuto'
        'portato'
        'upbow',
        'downbow'
        'flageolet'
        'thumb',
        'lheel'
        'rheel'
        'ltoe',
        'rtoe'
        'open'
        'stopped',
        'turn'
        'reverseturn'
        'trill',
        'prall'
        'mordent'
        'prallprall'
        'prallmordent',
        'upprall',
        'downprall',
        'upmordent',
        'downmordent',
        'pralldown',
        'prallup',
        'lineprall',
        'signumcongruentiae',
        'shortfermata',
        'fermata',
        'longfermata',
        'verylongfermata',
        'segno',
        'coda',
        'varcoda',
        '^',
        '+',
        '-',
        '|',
        '>',
        '.',
        '_',
        )

    _shortcut_to_word = {
        '^': 'marcato',
        '+': 'stopped',
        '-': 'tenuto',
        '|': 'staccatissimo',
        '>': 'accent',
        '.': 'staccato',
        '_': 'portato',
        }

    ### INITIALIZER ###

    def __init__(
        self,
        name: str = None,
        *,
        direction: typing.Union[str, enums.VerticalAlignment] = None,
        tweaks: typing.Union[
            typing.List[typing.Tuple], LilyPondTweakManager] = None,
        ) -> None:
        if isinstance(name, type(self)):
            argument = name
            name = argument.name
            direction = direction or argument.direction
        name = str(name)
        if '\\' in name:
            direction, name = name.split('\\')
            direction = direction.strip()
            name = name.strip()
        self._name = name
        direction_ = String.to_tridirectional_ordinal_constant(direction)
        if direction_ is not None:
            assert isinstance(direction_, enums.VerticalAlignment), repr(direction_)
            assert direction_ in (enums.Up, enums.Down, enums.Center), repr(direction_)
        self._direction = direction_
        self._format_slot = 'after'
        self._tweaks = None
        LilyPondTweakManager.set_tweaks(self, tweaks)

    ### SPECIAL METHODS ###

    def __format__(self, format_specification='') -> str:
        """
        Formats articulation.

        Set ``format_specification`` to `''`, `'lilypond`' or `'storage'`.
        Interprets `''` equal to `'storage'`.
        """
        if format_specification in ('', 'storage'):
            return StorageFormatManager(self).get_storage_format()
        else:
            assert format_specification == 'lilypond'
            return self._get_lilypond_format()

    def __illustrate__(self):
        """
        Illustrates articulation.

        Returns LilyPond file.
        """
        note = abjad.Note("c'4")
        articulation = copy.copy(self)
        attach(articulation, note)
        lilypond_file = abjad.LilyPondFile.new(note)
        return lilypond_file

    def __str__(self) -> str:
        """
        Gets string representation of articulation.
        """
        if self.name:
            string = self._shortcut_to_word.get(self.name)
            if not string:
                string = self.name
            if self.direction is None:
                direction = String('-')
            else:
                direction_ = String.to_tridirectional_lilypond_symbol(
                    self.direction)
                assert isinstance(direction_, String), repr(direction)
                direction = direction_
            return f'{direction}\{string}'
        else:
            return ''

    ### PRIVATE METHODS ###

    def _get_format_specification(self):
        values = [self.name]
        if self.direction is not None:
            values.append(self.direction)
        return FormatSpecification(
            client=self,
            storage_format_args_values=values,
            storage_format_is_indented=False,
            )

    def _get_lilypond_format(self):
        return str(self)

    def _get_lilypond_format_bundle(self, component=None):
        bundle = LilyPondFormatBundle()
        if self.tweaks:
            tweaks = self.tweaks._list_format_contributions()
            bundle.after.articulations.extend(tweaks)
        bundle.after.articulations.append(self._get_lilypond_format())
        return bundle

    ### PUBLIC PROPERTIES ###

    @property
    def direction(self) -> typing.Optional[enums.VerticalAlignment]:
        """
        Gets direction of articulation.

        ..  container:: example

            Without direction:

            >>> articulation = abjad.Articulation('staccato')
            >>> articulation.direction is None
            True

        ..  container:: example

            With direction:

            >>> articulation = abjad.Articulation('staccato', direction=abjad.Up)
            >>> articulation.direction
            Up

        """
        return self._direction

    @property
    def name(self) -> str:
        """
        Gets name of articulation.

        ..  container:: example

            Staccato:

            >>> articulation = abjad.Articulation('staccato')
            >>> articulation.name
            'staccato'

        ..  container:: example

            Tenuto:

            >>> articulation = abjad.Articulation('tenuto')
            >>> articulation.name
            'tenuto'

        """
        return self._name

    @property
    def tweaks(self) -> typing.Optional[LilyPondTweakManager]:
        r"""
        Gets tweaks

        ..  container:: example

            >>> note = abjad.Note("c'4")
            >>> articulation = abjad.Articulation('marcato')
            >>> abjad.tweak(articulation).color = 'blue'
            >>> abjad.attach(articulation, note)
            >>> abjad.show(note) # doctest: +SKIP

            ..  docs::

                >>> abjad.f(note)
                c'4
                - \tweak color #blue
                -\marcato

        ..  container:: example

            >>> note = abjad.Note("c'4")
            >>> articulation = abjad.Articulation(
            ...     'marcato',
            ...     tweaks=[('color', 'blue')],
            ...     )
            >>> abjad.attach(articulation, note)
            >>> abjad.show(note) # doctest: +SKIP

            ..  docs::

                >>> abjad.f(note)
                c'4
                - \tweak color #blue
                -\marcato

        """
        return self._tweaks
